(function () {
    'use strict';
    var app = angular.module('smnNg');
    app.directive('smnShell', [
        function () {
            return {
                restrict: 'A',
                templateUrl: 'assets/public/directives/main/shell.html',
                replace: true
            };
        }]);
    app.directive('smnGreeting', ['$location', '$timeout', 'notifier', 'common_service',
        function ($location, $timeout, notifier, common_service) {
            return {
                restrict: 'A',
                templateUrl: 'assets/public/directives/main/greeting.html',
                replace: true,
                link: function (scope) {

                    scope.greeting = {main_logo: undefined};
                    scope.greeting.main_logo = common_service.random_logo();
                    var notify = [
                        {i: 'Preparing the application...', t: 'info'}
                    ];
                    if (scope.right_nav !== undefined && scope.right_nav.pref !== undefined) {
                        if (scope.right_nav.pref.location === true) {
                            notify.push({i: 'Location: On', t: 'info'});
                        } else {
                            notify.push({i: 'Location: Off', t: 'warning'});
                        }
                        if (scope.right_nav.pref.hints === true) {
                            notify.push({i: 'Hints: On', t: 'info'});
                        } else {
                            notify.push({i: 'Hints: Off', t: 'warning'});
                        }

                    }
                    var i = 0, progress_bar;
                    angular.forEach(notify, function (v) {
                        $timeout(function () {
                            notifier.notify(v.i, v.t);
                        }, i === 0 ? 500 : (i * 1500));
                        i++;
                    });
                    progress_bar = angular.element('.progress-bar');

                    function finish() {
                        if ($location.path() === '/') {
                            $location.url('home');
                        }
                    }

                    function changeBg(color) {
                        angular.element('html, body').css('background-color', color);
                    }

                    function progress() {
                        var cb = common_service.random_number(500), counter = 0,
                            countDown = function () {
                                counter = counter + 1;
                                if (counter === 11) {
                                    $timeout(function () {
                                        finish();
                                    }, 500);
                                } else {
                                    progress_bar.width(counter * 10 + "%");
                                    $timeout(function () {
                                        countDown();
                                    }, cb);
                                }
                            };
                        countDown();
                    }

                    function setupBg() {

                        var background_one = common_service.random_colors(),
                            background_two = common_service.random_colors();

                        while (background_two === background_one) {
                            background_two = common_service.random_colors();
                        }
                        changeBg(background_one);
                        progress_bar.css('background-color', background_two);
                    }

                    (function () {
                        setupBg();
                        $timeout(function () {
                            progress_bar.width(common_service.random_number(10) + "%");
                            var cb = common_service.random_number(500);
                            $timeout(function () {
                                progress();
                            }, cb);
                        }, 1000);

                    }());
                }
            };
        }]);
    app.directive('smnSlider', ['common_service', function (common_service) {
        return {
            restrict: 'A',
            templateUrl: 'assets/public/directives/main/slider.html',
            replace: true,
            link: function (scope) {
                scope.slider = {img: []};
                var path = 'assets/img/site/slider/', i = 1;
                if (common_service.is_mobile_device()) {
                    path = path + 'port/';
                } else if (!common_service.is_mobile_device()) {
                    path = path + 'desk/';
                }
                for (i; i < 8; i = (i + 1)) {
                    scope.slider.img.push(path + 'smn' + i + '.jpg');
                }
            }
        };
    }]);
    app.directive('smnSlide', ['$timeout', function ($timeout) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                $timeout(function () {
                    elem.superslides(
                        {
                            play: 7000,
                            animation: 'fade',
                            animation_speed: 'slow',
                            animation_easing: 'linear',
                            pagination: false,
                            hashchange: false
                        }
                    );
                }, 10);
            }
        };

    }]);

    app.directive('smnViewHandlerDefault', ['$timeout', 'jQuery', function ($timeout, jQuery) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                scope.$on('$routeChangeStart', function (next, current) {
                    if (current.$$route) {
                        if (current.$$route.originalPath === '/home') {
                            $timeout(function () {
                                elem.fadeIn(2000);
                            }, 2000);
                            jQuery('html, body').addClass('html-body');
                        } else if (current.$$route.originalPath === '/news') {
                            $timeout(function () {
                                elem.fadeIn(2000);
                            }, 2000);
                            $timeout(function () {
                                jQuery('html, body').removeClass('html-body');
                            }, 8000);
                        } else if (current.$$route.originalPath === '/news/:id') {
                            jQuery('html, body').addClass('html-body');
                        } else if (current.$$route.originalPath === '/') {
                            elem.hide();
                            jQuery('html, body').addClass('html-body');
                        }
                    }
                });
                scope.$on('$routeChangeSuccess', function (event, current) {
                    if (current.$$route) {
                        if (current.$$route.originalPath === '/home') {
                            $timeout(function () {
                                elem.fadeIn(2000);
                            }, 2000);
                        } else if (current.$$route.originalPath === '/news') {
                            $timeout(function () {
                                elem.fadeIn(2000);
                            }, 2000);
                        } else if (current.$$route.originalPath === '/') {
                            elem.hide();
                        }
                    }
                });
            }
        };
    }]);

    app.directive('smnViewHandlerNews', ['$timeout', function ($timeout) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                scope.$on('$routeChangeStart', function (next, current) {
                    if (current.$$route) {
                        if (current.$$route.originalPath === '/home') {
                            elem.hide();
                        } else if (current.$$route.originalPath === '/news') {
                            elem.hide();
                        } else if (current.$$route.originalPath === '/') {
                            elem.hide();
                        }
                    }
                });
                scope.$on('$routeChangeSuccess', function (event, current) {
                    if (current.$$route) {
                        if (current.$$route.originalPath === '/home') {
                            elem.hide();
                        } else if (current.$$route.originalPath === '/news') {
                            $timeout(function () {
                                elem.fadeIn(2000);
                            }, 2000);
                        } else if (current.$$route.originalPath === '/') {
                            elem.hide();
                        }
                    }
                });
            }
        };
    }]);

    app.directive('smnNavViewHandlerHome', [function () {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                scope.$on('$routeChangeStart', function (event, next, cureent) {
                });
                scope.$on('$routeChangeSuccess', function (event, current) {
                    if (current.$$route) {
                        if (current.$$route.originalPath === '/home') {
                            elem.show();
                        } else if (current.$$route.originalPath === '/news') {
                            elem.hide();
                        } else if (current.$$route.originalPath === '/news/:id') {
                            elem.show();
                        }
                    }
                });
            }
        };
    }]);
    app.directive('smnNavViewHandlerArticle', [function () {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                scope.$on('$routeChangeStart', function (event, next, cureent) {
                });
                scope.$on('$routeChangeSuccess', function (event, current) {
                    if (current.$$route) {
                        if (current.$$route.originalPath === '/news/:id') {
                            elem.hide();
                        } else if (current.$$route.originalPath === '/news') {
                            elem.show();
                        }
                    }
                });
            }
        };
    }]);
    app.directive('smnNavViewHandlerNews', [function () {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                scope.$on('$routeChangeStart', function (event, next, cureent) {
                });
                scope.$on('$routeChangeSuccess', function (event, current) {
                    if (current.$$route) {
                        if (current.$$route.originalPath === '/home') {
                            elem.hide();
                        } else if (current.$$route.originalPath === '/news') {
                            elem.show();
                        }
                    }
                });
            }
        };
    }]);
    app.directive('smnPortableOnly', ['common_service', function (cv) {
        return {
            restrict: 'A',
            replace: true,
            link: function (s, elem) {
                if (!cv.is_mobile_device() || !cv.is_touch_device()) {
                    elem.remove();
                }
            }
        };
    }]);
    app.directive('smnTooltip', [ function () {
        return {
            restrict: 'A',
            replace: true,
            scope: {},
            link: function (s, elem) {
                elem.tooltip();
            }
        };
    }]);
    app.directive('smnDesktopOnly', ['common_service', function (cv) {
        return {
            restrict: 'A',
            replace: true,
            link: function (s, elem) {
                if (cv.is_mobile_device() || cv.is_touch_device()) {
                    elem.remove();
                }
            }
        };
    }]);
    app.directive('smnOnDown', [function () {
        return {
            restrict: 'A',
            replace: true,
            link: function (s, elem, attr) {
                elem.on('keyup', function (e, v) {
                });
            }
        };
    }]);

    app.directive('smnBtPgBroadcast', ['$document', '$window', '$routeParams', function ($document, $window, $routeParams) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope) {
                angular.element($window).bind('scroll', function () {
                    if (angular.element($window).scrollTop() + angular.element($window).height() >= angular.element($document).height()) {
                        if ($routeParams.id === undefined &&
                            scope.right_nav !== undefined &&
                            scope.right_nav.pref !== undefined &&
                            scope.right_nav.pref.art_view !== 'world' &&
                            scope.n_ctrl.division !== undefined &&
                            scope.n_ctrl.updating === false &&
                            scope.n_ctrl.modal === false) {
                            scope.$broadcast('end_page', function () {
                                return 1;
                            });

                        }
                    }
                });
            }
        };
    }]);

    app.directive('smnTheme', ['preferences', function (pref) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                function run() {
                    var setting = pref.xget_pref();
                    if (setting !== undefined && setting.nav_style !== undefined) {
                        elem.addClass(setting.nav_style);
                    }
                }

                run();
                scope.$on('user_prefs', function () {
                    run();
                })
            }
        };
    }]);

    app.directive('smnSpin', ['common_service', function (cv) {
        return {
            restrict: 'A',
            replace: true,
            link: function (s, elem) {
                elem.spin({lines: 13, length: 1, width: 10, radius: 30, corners: 1, rotate: 0, direction: 1, speed: 1,
                    color: cv.random_colors(), trail: 80, shadow: true, hwaccel: false, zIndex: 2e9});
            }
        };
    }]);
    app.directive('smnRandomColor', ['common_service', function (cv) {
        return {
            restrict: 'A',
            replace: true,
            scope: {type: '='},
            link: function (s, elem) {
                var c = cv.random_colors();
                if (s.type === 1) {
                    elem.css('color', c);
                } else {
                    elem.css('background-color', c);
                }

            }
        };
    }]);
    app.directive('smnBackgroundColor', ['jQuery', function (jQuery) {
        return {
            restrict: 'A',
            replace: true,
            scope: {where: '='},
            link: function (s) {
                function add(c) {
                    jQuery('body').addClass(c);
                }

                function clean() {
                    jQuery('body').removeClass('a-bg-t m-bg-t m2-bg-t m3-bg-t m4-bg-t');
                }

                clean();
                if (s.where === 0) {
                    add('m-bg-t');
                } else if (s.where === 1) {
                    add('a-bg-t');
                } else if (s.where === 2) {
                    add('m2-bg-t');
                } else if (s.where === 3) {
                    add('m3-bg-t');
                } else if (s.where === 4) {
                    add('m4-bg-t');
                } else if (s.where === -1) {
                }
            }
        };
    }]);
    app.directive('smnScaleThis', [function () {
        return {
            restrict: 'A',
            replace: true,
            link: function (s, elem) {

                elem.hover(function () {
                    elem.toggleClass('smn-s-i');
                });
                elem.focus(function () {
                    elem.toggleClass('smn-s-i');
                });
            }
        };
    }]);
    app.directive('smnScaleThat', [function () {
        return {
            restrict: 'A',
            replace: true,
            link: function (s, elem) {

                elem.hover(function () {
                    elem.toggleClass('smn-s-ii');
                });
                elem.focus(function () {
                    elem.toggleClass('smn-s-ii');
                });
            }
        };
    }]);

    app.directive('smnHelp', ['preferences', 'notifier', function (pref, notifier) {
        return {
            restrict: 'A',
            replace: true,
            scope: {
                note: '=',
                multi: '=',
                ftype: '='
            },
            link: function (scope, elem) {
                var c = false, hints;
                elem.click(function (event) {
                    event.preventDefault();
                    event.stopPropagation();
                    hints = pref.xget_pref();
                    if (hints !== undefined && hints.hints === true && c === false) {
                        notifier.notify(scope.note, scope.ftype);
                        c = true;
                        if (scope.multi !== undefined) {
                            c = false;
                        }
                    }
                });
            }
        };
    }]);
    app.directive('smnImg', [function () {
        /* <img data-smn-img={{url}} >*/
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem, attrs) {
                attrs.$observe('smnImg', function (value) {
                    if (value === undefined || value.length === 0 || !value) {
                        value = 'assets/img/site/default/notf.jpg';
                    }
                    attrs.$set('src', value);
                });
                elem.bind('error', function () {
                    elem.attr('src', 'assets/img/site/default/notf.jpg');
                });
            }
        };
    }]);
    app.directive('smnSpinThatOnUpdate', [function () {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                scope.$on('up_news_s', function () {
                    elem.addClass('no-borders fa-spin');
                });
                scope.$on('up_news_e', function () {
                    elem.removeClass('no-borders fa-spin');
                });
            }
        };
    }]);
    app.directive('smnPushDownSlow', ['$timeout', '$anchorScroll', function ($timeout, $anchorScroll) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                $anchorScroll();
                $timeout(function () {
                    elem.addClass('top-row');
                }, 1500);
            }
        };
    }]);
    app.directive('smnNormalizeSpacing', ['$timeout', function ($timeout) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                $timeout(function () {
                    elem.removeClass('sick');
                }, 1500);
            }
        };
    }]);
    app.directive('smnSetUpLoad', ['$timeout', 'jQuery', function ($timeout, jQuery) {
        return {
            restrict: 'A',
            replace: true,
            link: function (scope, elem) {
                function addHeight() {
                    jQuery('html, body').css('height', '100%');
                }

                function removeHeight() {
                    jQuery('html, body').css('height', '');
                }

                scope.$on('$routeChangeStart', function (next, current) {
                    if (current.$$route) {
                        if (current.$$route.originalPath === '/') {
                            addHeight();
                        } else if (current.$$route.originalPath === '/home') {
                            addHeight();
                        } else if (current.$$route.originalPath === '/news') {
                            addHeight();
                        } else if (current.$$route.originalPath === '/news/:id') {
                            addHeight();
                        }
                    }
                });
                scope.$on('ch_view', function () {
                    addHeight();
                });
                scope.$on('ch_cp', function () {
                    $timeout(function () {
                        removeHeight();
                    }, 2500);
                });

            }
        };
    }]);
    app.filter("breaker", ['common_service', function (cv) {
        return function (data) {
            if (data !== undefined) {
                if (cv.is_touch_device() && cv.is_mobile_device()) {
                    return data.replace(/\n\r?/g, '<br/><br/>');
                }
                return data.replace(/\n\r?/g, '<br/>');
            }
            return data;
        };
    }]);
    app.directive('smnLeftNav', ['$timeout', 'menu', 'preferences', 'jQuery', 'modal_manager',
        function ($timeout, menu, pref, jQuery, mm) {
            return {
                restrict: 'A',
                templateUrl: 'assets/public/directives/nav/leftnav.html',
                replace: true,
                controller: function ($scope) {
                    $scope.$on('$routeChangeStart', function (next, current) {
                        if (current.$$route) {
                            mm.menu_op($scope);
                        }
                    });
                    $scope.show_auth_modal = function () {
                        var wait = 100;
                        if (jQuery('.smnlfnav').hasClass('nav-open')) {
                            $scope.left_btn.toggle_left_nav();
                            menu.toggleLeft();
                            wait = 700;
                        }
                        $timeout(function () {
                            jQuery('#auth-modal').modal('show');
                            $timeout(function () {
                                jQuery('#imail').focus();
                            }, 500);
                        }, wait);
                    };
                },
                link: function (scope) {
                    scope.left_nav = {pref: undefined, show_auth_modal: scope.show_auth_modal, allow_hints: undefined,
                        change_style: undefined, col: {theme: true, link: true, hints: true}};

                    function set_app_pres() {
                        scope.left_nav.pref = pref.xget_pref();
                        if (scope.left_nav.pref === undefined) {
                            scope.left_nav.pref = {hints: false, nav_style: 'default'};
                            pref.xset_pref(scope.left_nav.pref);
                        }
                    }

                    set_app_pres();
                    scope.left_nav.change_style = function (style) {
                        scope.left_nav.pref.nav_style = style;
                        pref.xset_pref(scope.left_nav.pref);

                        var el = jQuery('.smnlfnav, .smnrtnav, .smn-border, .search-bg, #left-menu-toggle, #right-menu-toggle, .customize-nav');
                        if (style === 'default') {
                            el.removeClass('theme-smn theme-retro theme-ios theme-android');
                        } else if (style === 'theme-ios') {
                            el.removeClass('theme-smn theme-retro theme-android').addClass('theme-ios');
                        } else if (style === 'theme-android') {
                            el.removeClass('theme-smn theme-retro theme-ios').addClass('theme-android');
                        } else if (style === 'theme-retro') {
                            el.removeClass('theme-smn theme-ios theme-android').addClass('theme-retro');
                        } else if (style === 'theme-smn') {
                            el.removeClass('theme-ios theme-android theme-retro').addClass('theme-smn');

                        }
                        scope.left_nav.col.theme = true;
                    };

                    scope.left_nav.allow_hints = function () {

                        scope.left_nav.pref.hints = !scope.left_nav.pref.hints;
                        pref.xset_pref(scope.left_nav.pref);
                        scope.left_nav.col.hints = true;
                    };

                    scope.left_nav.open_cl = function (cl) {
                        if (cl === 'theme') {
                            scope.left_nav.col.theme = !scope.left_nav.col.theme;
                            scope.left_nav.col.link = true;
                            scope.left_nav.col.hints = true;
                        } else if (cl === 'link') {
                            scope.left_nav.col.link = !scope.left_nav.col.link;
                            scope.left_nav.col.theme = true;
                            scope.left_nav.col.hints = true;
                        } else {
                            scope.left_nav.col.hints = !scope.left_nav.col.hints;
                            scope.left_nav.col.theme = true;
                            scope.left_nav.col.link = true;
                        }
                    };
                    scope.$on('user_prefs', function () {
                        set_app_pres();
                    });
                }
            };
        }]);
    app.directive('smnRightNav', ['preferences', '$rootScope', '$q', '$routeParams', 'modal_manager', 'jQuery', '$timeout',
        function (pref, $rootScope, $q, $routeParams, mm, jQuery, $timeout) {
            return {
                restrict: 'A',
                templateUrl: 'assets/public/directives/nav/rightnav.html',
                replace: true,
                controller: function ($scope) {
                    $scope.$on('$routeChangeStart', function (next, current) {
                        if (current.$$route) {
                            mm.menu_op($scope);
                        }
                    });
                },
                link: function (scope) {
                    scope.right_nav = {col: {cat: true, art: true, news: true, loc: true},
                        change_opening: undefined, allow_location: undefined, change_art_view: undefined, update: undefined,
                        search: undefined, filter: undefined, updating: false, pref: undefined, op_src: undefined};
                    function on_auth_active() {
                        var defer = $q.defer();
                        scope.$on('proceed', function () {
                            defer.resolve(1);
                        });
                        return defer.promise;
                    }

                    function set_app_pref() {
                        scope.right_nav.pref = pref.xget_pref();
                        if (scope.right_nav.pref.opening === undefined) {
                            scope.right_nav.pref.opening = 'modal';
                        }
                        if (scope.right_nav.pref.location === undefined) {
                            scope.right_nav.pref.location = false;

                        }
                        if (scope.right_nav.pref.location === true && pref.get_geo() === undefined) {
                            on_auth_active().then(function () {
                                scope.auth.set_location();
                            });
                        }
                        if (scope.right_nav.pref.art_view === undefined) {
                            scope.right_nav.pref.art_view = 'gart';
                        }
                        if (scope.right_nav.pref.fsize === undefined) {
                            scope.right_nav.pref.fsize = 3;
                        }
                        if (scope.right_nav.pref.bcolor === undefined) {
                            scope.right_nav.pref.bcolor = 1;
                        }
                        if (scope.right_nav.pref.afont === undefined) {
                            scope.right_nav.pref.afont = 'n';
                        }
                        pref.xset_pref(scope.right_nav.pref);
                    }

                    set_app_pref();
                    scope.$on('user_prefs', function () {
                        set_app_pref();
                    });
                    scope.right_nav.change_opening = function (opening) {
                        scope.right_nav.pref.opening = opening;
                        pref.xset_pref(scope.right_nav.pref);
                        scope.right_nav.col.art = true;
                    };
                    scope.right_nav.allow_location = function () {
                        scope.right_nav.pref.location = !scope.right_nav.pref.location;
                        pref.xset_pref(scope.right_nav.pref);
                        scope.right_nav.col.loc = true;
                        if (!scope.right_nav.pref.location) {
                            pref.delete_geo('geo');
                            scope.$broadcast('delete_geo', function () {
                                return 1;
                            });
                        } else {
                            scope.$broadcast('get_location', function () {
                                return 1;
                            });
                            scope.auth.set_location();
                        }
                    };
                    scope.right_nav.change_art_view = function (view) {
                        $rootScope.$broadcast('ch_view', function () {
                            return 1;
                        });
                        scope.right_nav.filter = '';
                        scope.right_nav.pref.art_view = view;
                        pref.xset_pref(scope.right_nav.pref);
                        scope.right_nav.col.news = true;
                    };
                    scope.right_nav.open_cl = function (cl) {
                        if (cl === 'cat') {
                            scope.right_nav.col.cat = !scope.right_nav.col.cat;
                            scope.right_nav.col.art = true;
                            scope.right_nav.col.news = true;
                            scope.right_nav.col.loc = true;
                        } else if (cl === 'art') {
                            scope.right_nav.col.art = !scope.right_nav.col.art;
                            scope.right_nav.col.cat = true;
                            scope.right_nav.col.news = true;
                            scope.right_nav.col.loc = true;
                        } else if (cl === 'news') {
                            scope.right_nav.col.news = !scope.right_nav.col.news;
                            scope.right_nav.col.art = true;
                            scope.right_nav.col.cat = true;
                            scope.right_nav.col.loc = true;
                        } else {
                            scope.right_nav.col.loc = !scope.right_nav.col.loc;
                            scope.right_nav.col.art = true;
                            scope.right_nav.col.news = true;
                            scope.right_nav.col.cat = true;
                        }

                    };
                    function update(a) {
                        scope.$broadcast('up_news_s', function () {
                            return a;
                        });
                    }

                    scope.right_nav.update = function () {
                        $rootScope.$broadcast('ch_view', function () {
                            return 1;
                        });
                        if (scope.right_nav.updating === false) {
                            if ($routeParams.id !== undefined) {
                                update(0);
                            } else {
                                update(1);
                                scope.right_nav.updating = true;
                            }
                        }
                    };
                    scope.$on('up_news_e', function () {
                        scope.right_nav.updating = false;
                    });
                    scope.$on('manual_geo', function () {
                        scope.right_nav.allow_location();
                    });

                    (function activate() {
                        $rootScope.$broadcast('right_nav', function () {
                            return 1;
                        });
                    }());
                    scope.right_nav.s_class = function () {
                        if (scope.right_nav.filter !== undefined) {
                            if (scope.right_nav.filter.length <= 3) {
                                return '';
                            } else if (scope.right_nav.filter.length > 3) {
                                return 'fa-spin';
                            }
                        } else {
                            return '';
                        }
                    };
                    scope.right_nav.search = function () {
                        $rootScope.$broadcast('filter', function () {
                            return scope.right_nav.filter;
                        });

                    };
                    scope.right_nav.op_src = function () {
                        mm.op_src(scope);
                        $rootScope.$broadcast('filter_m', function () {
                            return 1;
                        });
                    };
                }
            };
        }]);
    app.directive('smnLeftNavBtn', ['menu', 'common_service', 'jQuery', function (menu, common_service, jQuery) {
        return {
            restrict: 'A',
            replace: true,
            controller: function ($scope) {
                $scope.left_btn = {click: false, toggle_left_nav: undefined};
                $scope.left_btn.toggle_left_nav = function () {
                    var btn = jQuery('#left-menu-toggle');
                    if (!btn.hasClass('btn-close')) {
                        btn.children('i').removeClass('fa-align-justify');
                        btn.children('i').addClass('fa-align-right');
                        if (common_service.is_mobile_device() || common_service.is_touch_device()) {
                            jQuery('body').removeClass('hide-overflow');
                        }
                        $scope.left_btn.click = false;

                    } else if (btn.hasClass('btn-close')) {
                        btn.children('i').removeClass('fa-align-right');
                        btn.children('i').addClass('fa-align-justify');
                        if (common_service.is_mobile_device() || common_service.is_touch_device()) {
                            jQuery('body').addClass('hide-overflow');
                        }
                        $scope.left_btn.click = true;
                    }
                    btn.toggleClass('btn-close');
                };
            },
            link: function (scope, elem) {
                if (common_service.is_transform3d()) {
                    elem.click(function (event) {
                        event.stopPropagation();
                        scope.left_btn.toggle_left_nav();
                        menu.toggleLeft();
                    });
                }
            }
        };
    }]);

    app.directive('smnRightNavBtn', ['menu', 'common_service', 'jQuery',
        function (menu, common_service, jQuery) {
            return {
                restrict: 'A',
                replace: true,
                controller: function ($scope) {
                    $scope.right_btn = {click: false, toggle_right_nav: undefined};
                    $scope.right_btn.toggle_right_nav = function () {
                        var btn = jQuery('#right-menu-toggle');
                        if (!btn.hasClass('btn-close')) {
                            btn.children('i').removeClass('fa-gears');
                            btn.children('i').addClass('fa-gear');
                            if (common_service.is_mobile_device() || common_service.is_touch_device()) {
                                jQuery('body').removeClass('hide-overflow');
                            }
                            $scope.right_btn.click = false;
                        } else if (btn.hasClass('btn-close')) {
                            btn.children('i').removeClass('fa-gear');
                            btn.children('i').addClass('fa-gears');
                            if (common_service.is_mobile_device() || common_service.is_touch_device()) {
                                jQuery('body').addClass('hide-overflow');
                            }
                            $scope.right_btn.click = true;

                        }
                        btn.toggleClass('btn-close');
                    };
                },
                link: function (scope, elem) {
                    if (common_service.is_transform3d()) {
                        elem.click(function (event) {
                            event.stopPropagation();
                            scope.right_btn.toggle_right_nav();
                            menu.toggleRight();
                        });
                    }
                }
            };
        }]);

    app.directive('smnSwipeLeft', ['common_service', 'menu',
        function (common_service, menu) {
            return {
                restrict: 'A',
                replace: true,
                link: function (scope, elem) {
                    scope.swipe_left = {swipeRightToOpen: undefined, swipeLeftToClose: undefined};
                    if (common_service.is_mobile_device() || common_service.is_touch_device()) {
                        /*elem.addClass('drag-shadow-left');*/
                    }
                    scope.swipe_left.swipeRightToOpen = function () {
                        if (!scope.left_btn.click) {
                            scope.left_btn.toggle_left_nav();
                            menu.toggleLeft();
                            scope.left_btn.click = true;
                        }
                    };
                    scope.swipe_left.swipeLeftToClose = function () {
                        if (scope.left_btn.click) {
                            scope.left_btn.toggle_left_nav();
                            menu.toggleLeft();
                            scope.left_btn.click = false;
                        }
                    };
                }
            };
        }]);
    app.directive('smnSwipeRight', ['common_service', 'menu',
        function (common_service, menu) {
            return {
                restrict: 'A',
                replace: true,
                link: function (scope, elem) {
                    scope.swipe_right = {swipeLeftToOpen: undefined, swipeRightToClose: undefined};
                    if (common_service.is_mobile_device() || common_service.is_touch_device()) {
                        /*elem.addClass('drag-shadow-right');*/
                    }
                    scope.swipe_right.swipeLeftToOpen = function () {
                        if (!scope.right_btn.click) {
                            scope.right_btn.toggle_right_nav();
                            menu.toggleRight();
                            scope.right_btn.click = true;
                        }
                    };
                    scope.swipe_right.swipeRightToClose = function () {
                        if (scope.right_btn.click) {
                            scope.right_btn.toggle_right_nav();
                            menu.toggleRight();
                            scope.right_btn.click = false;
                        }
                    };
                }
            };
        }]);
    app.directive('smnAuth', ['$timeout', 'animation', 'jQuery', 'notifier', 'user_manager', 'preferences', 'menu', '$rootScope', 'page_manager',
        function ($timeout, animation, jQuery, notifier, user_manager, pref, menu, $rootScope, pm) {
            return {
                restrict: 'A',
                templateUrl: 'assets/public/directives/modal/auth.html',
                replace: true,
                link: function (scope) {
                    scope.auth = {auth: undefined, register: undefined, log_out: undefined, set_location: undefined,
                        show_profile: undefined, identity: undefined, location: undefined};
                    scope.auth.identity = user_manager.identify_user();

                    function start_request(elem) {
                        animation.toggle(elem);
                    }

                    function start_sm_request(elem, size) {
                        animation.toggle(elem, undefined, size);
                    }

                    function end_request(form, elem, addition, error) {
                        if (error === 0) {
                            form.$setPristine();
                            form.email = undefined;
                            form.password = undefined;
                        }
                        animation.toggle(elem, addition);
                    }

                    scope.auth.set_location = function () {

                        var allow = pref.xget_pref();
                        if (allow.location) {
                            user_manager.current_loc().then(function (location) {
                                pref.set_geo(location);
                                scope.$broadcast('location_on', function () {
                                    return 1;
                                });
                            });
                        }
                    };
                    function handle_request(result, form, elem, addition, action) {

                        if (result !== undefined) {
                            scope.auth.set_location();
                            $timeout(function () {
                                jQuery('#auth-modal').modal('hide');
                                end_request(form, elem, addition, 0);
                                $timeout(function () {
                                    if (action === 0) {
                                        notifier.notify('You have successfully signed in', 'success');
                                    } else if (action === 1) {
                                        notifier.notify('You have successfully registered and signed in', 'success');
                                    }
                                    scope.$broadcast('proceed', function () {
                                        return 1;
                                    });
                                }, 500);
                            }, 1500);
                        } else if (result === undefined) {
                            $timeout(function () {
                                end_request(form, elem, 'Try again', 1);
                                form.$valid = true;
                                if (action === 0) {
                                    notifier.notify('Wrong username or password', 'error');
                                } else if (action === 1) {
                                    notifier.notify('This email is already registered, please sing in', 'error');
                                } else if (action === -1) {
                                    notifier.notify('Something went very wrong', 'error');
                                }
                            }, 1500);
                            return 1;
                        }
                    }

                    function declare_identity(result) {
                        if (result !== undefined) {
                            scope.auth.identity = user_manager.identify_user();
                        }
                    }

                    scope.auth.auth = function (form) {
                        if (form !== undefined && form.$valid) {
                            if (form.email.length >= 7 && form.password.length >= 5) {
                                form.$valid = false;
                                start_request('#isub');
                                var user = {email: form.email, password: form.password}
                                user_manager.authenticate_user(user).
                                    then(function (result) {
                                        declare_identity(result);
                                        handle_request(result, form, '#isub', 'Sign In', 0);
                                        if (result !== undefined) {
                                            user.uid = result.uid;
                                            user_manager.get_user_prefs(user).then(function (r) {
                                                if (r !== undefined && r.prefs !== undefined && Object.keys(r.prefs).length >= 8) {
                                                    pref.xset_pref(r.prefs);
                                                    $rootScope.$broadcast('user_prefs', function () {
                                                        return 1;
                                                    });
                                                }
                                            });
                                        }
                                    }, function () {
                                        handle_request(undefined, form, '#isub', 'Sign In', -1);
                                    }
                                );
                            }
                        }
                    };
                    scope.auth.register = function (form) {
                        if (form !== undefined && form.$valid) {
                            if (form.email.length >= 7 && form.password.length >= 5) {
                                form.$valid = false;
                                start_request('#usub');
                                user_manager.register_user({email: form.email, password: form.password}).
                                    then(function (result) {
                                        declare_identity(result);
                                        handle_request(result, form, '#usub', 'Sign Up', 1);
                                    }, function () {
                                        handle_request(undefined, form, '#usub', 'Sign Up', -1);
                                    }
                                );
                            }
                        }
                    };
                    scope.auth.log_out = function (action) {
                        user_manager.set_user_prefs(scope.auth.identity, pref.xget_pref());
                        var wait = 1500;
                        if (action === 1) {
                            start_sm_request('#alogout', 1);
                        } else {
                            animation.anima_i('#lflo', '', 'no-borders fa-spin fa-spinner');
                        }
                        user_manager.logout_user().
                            then(function (result) {
                                if (result === true) {
                                    if (pm.page_id() !== undefined) {
                                        pm.navigate('news');
                                    }
                                    $timeout(function () {
                                        scope.auth.identity = undefined;
                                        notifier.notify('You have successfully logged out', 'warning');
                                    }, wait);
                                }
                            }
                        );
                    };
                    scope.auth.show_profile = function () {

                        notifier.notify('Getting profile...');
                        user_manager.get_user_profile(scope.auth.identity).then(
                            function (response) {
                                scope.profile = response.profile;
                                scope.auth.location = pref.get_geo();
                                var wait = 100;
                                if (jQuery('.smnlfnav').hasClass('nav-open')) {
                                    scope.left_btn.toggle_left_nav();
                                    menu.toggleLeft();
                                    wait = 700;
                                }
                                $timeout(function () {
                                    jQuery('#profile-modal').modal('show');
                                    jQuery('#pfname').focus();
                                }, wait);
                            }
                        );
                    };
                    (function proceed() {
                        $rootScope.$broadcast('proceed', function () {
                            return 1;
                        });
                    }());
                }
            };
        }]);

    app.directive('smnFlipper', ['jQuery', 'common_service', '$timeout', function (jQuery, common_service, $timeout) {
        return {
            restrict: 'A',
            replace: true,
            scope: { xplace: '='},
            link: function (scope, elem) {
                var formContainer, fid, id;
                if (scope.xplace === 0) {
                    fid = '#formContainer';
                    id = '#umail';
                } else if (scope.xplace === 1) {
                    fid = '#formContainer';
                    id = '#imail';
                } else {
                    fid = '#xformContainer';
                    id = '#pfname';
                }
                elem.click(function (event) {
                    event.preventDefault();
                    formContainer = jQuery(fid);
                    formContainer.toggleClass('flipped');
                    if (!common_service.is_transform3d()) {
                        jQuery('#login').toggle();
                    }
                    $timeout(function () {
                        jQuery(id).focus();
                    }, 1000);
                });
            }
        };
    }]);

    app.directive('smnProfile', ['$timeout', 'animation', 'user_manager', 'constant_service', 'notifier',
        function ($timeout, animation, user_manager, cs, notifier) {
            return {
                restrict: 'A',
                templateUrl: 'assets/public/directives/modal/profile.html',
                replace: true,
                link: function (scope) {


                    scope.pf = {switch_m: undefined, save: undefined, col: {inf: false, edi: true}};

                    scope.days = cs.numbers(1, 31);
                    scope.months = cs.numbers(1, 12);
                    scope.years = cs.numbers(1900, 2000);
                    scope.occ = cs.app_occ();

                    scope.pf.switch_m = function (mode) {

                        if (mode === 1) {
                            scope.pf.col.inf = !scope.pf.col.inf;
                            $timeout(function () {
                                scope.pf.col.edi = !scope.pf.col.edi;
                            }, 300);
                        } else {
                            scope.pf.col.edi = !scope.pf.col.edi;
                            $timeout(function () {
                                scope.pf.col.inf = !scope.pf.col.inf;
                            }, 300);
                        }
                    };
                    function set_bd(p) {
                        return p.day + '/' + p.month + '/' + p.year;
                    }

                    scope.pf.save = function (form, profile) {
                        if (form.$valid) {
                            animation.toggle('#pfsub', undefined, 1);
                            notifier.notify('Updating profile...', 'warning');
                            profile.birthday.bd = set_bd(profile.birthday);
                            user_manager.set_user_profile(scope.auth.identity, profile).
                                then(function (result) {
                                    if (result === 1) {
                                        notifier.notify('Your profile is now updated');
                                        $timeout(function () {
                                            scope.pf.switch_m();
                                            $timeout(function () {
                                                animation.toggle('#pfsub', 'SAVE');
                                            }, 100);
                                        }, 300);
                                    }
                                });
                        }
                    };
                }
            };
        }]);
    app.directive('smnStatusModal', [ 'modal_manager',
        function (mm) {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/modal/status.html',
                link: function (scope) {
                    scope.yes = function () {
                        mm.cl_stat();
                    };
                }
            };
        }]);
    app.directive('smnMainView', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/view/main.html',
                scope: {articles: '=', xopen: '=', hd: '=', xtrim: '='}
            };
        }]);
    app.directive('smnMainViewSecond', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/view/main2.html',
                scope: {articles: '=', xopen: '=', hd: '=', xtrim: '='}
            };
        }]);
    app.directive('smnNewsCoolPart', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                scope: {articles: '=', xtrim: '=', xopen: '='},
                templateUrl: 'assets/public/directives/news/parts/cool_part.html',
                link: function (scope) {
                    scope.cool_filter = '';
                    scope.$on('filter', function (e, s) {
                        scope.cool_filter = s();
                    });
                }
            };
        }]);
    app.directive('smnNewsCoolView', [function () {
        return {
            restrict: 'A',
            replace: false,
            templateUrl: 'assets/public/directives/news/view/cool.html',
            scope: {articles: '=', xtrim: '=', xopen: '='}
        };
    }]);
    app.directive('smnNewsImgPart', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/parts/img_part.html',
                scope: {articles: '=', xopen: '='},
                link: function (scope) {
                    scope.img_filter = '';
                    scope.$on('filter', function (e, s) {
                        scope.img_filter = s();
                    });
                }
            };
        }]);
    app.directive('smnNewsImgView', [function () {
        return {
            restrict: 'A',
            replace: false,
            templateUrl: 'assets/public/directives/news/view/img.html',
            scope: {articles: '=', xopen: '='}
        };
    }]);
    app.directive('smnNewsGartPart', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/parts/gart_part.html',
                scope: {articles: '=', xopen: '=', xtrim: "="},
                link: function (scope) {
                    scope.gart_filter = '';
                    scope.$on('filter', function (e, s) {
                        scope.gart_filter = s();
                    });
                }
            };
        }]);
    app.directive('smnNewsGartView', [function () {
        return {
            restrict: 'A',
            replace: false,
            templateUrl: 'assets/public/directives/news/view/gart.html',
            scope: {articles: '=', xopen: '=', xtrim: "="}
        };
    }]);
    app.directive('smnNewsDetailsPart', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/parts/detail_part.html',
                scope: {articles: '=', xtrim: "=", xopen: '='},
                link: function (scope) {
                    scope.d_filter = '';
                    scope.$on('filter', function (e, s) {
                        scope.d_filter = s();
                    });
                }
            };
        }]);
    app.directive('smnNewsDetailsView', [function () {
        return {
            restrict: 'A',
            replace: false,
            templateUrl: 'assets/public/directives/news/view/detail.html',
            scope: {articles: '=', xtrim: "=", xopen: '='}
        };
    }]);
    app.directive('smnNewsCarouselPart', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/parts/carousel.html',
                scope: {hd: '=', xtrim: '=', xopen: '='}
            };
        }]);
    app.directive('smnNewsSpinPart', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/parts/spin.html'
            };
        }]);
    app.directive('smnNewsLoadingPart', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/parts/loading.html'
            };
        }]);
    app.directive('smnErrorArticlePage', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/article/error.html'
            };
        }]);
    app.directive('smnArticleModal', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                scope: {art: '=', xbreaks: '=', xtrim: '=', xflag: '='},
                templateUrl: 'assets/public/directives/news/article/modal.html'
            };
        }]);
    app.directive('smnArticleTagsList', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                scope: { tags: '='},
                templateUrl: 'assets/public/directives/news/article/tags.html'
            };
        }]);
    app.directive('smnArticleContent', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                scope: { art: '=', xbreaks: '=', xtrim: '='},
                templateUrl: 'assets/public/directives/news/article/content.html'
            };
        }]);
    app.directive('smnNewsListPart', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/parts/list_part.html',
                scope: {articles: '=', xopen: '='},
                link: function (scope) {
                    scope.li_filter = '';
                    scope.$on('filter', function (e, s) {
                        scope.li_filter = s();
                    });
                }
            };
        }]);
    app.directive('smnNewsListView', [function () {
        return {
            restrict: 'A',
            replace: false,
            templateUrl: 'assets/public/directives/news/view/list.html',
            scope: {articles: '=', xopen: '='}
        };
    }]);
    app.directive('smnArticleRater', [
        function () {
            return {
                restrict: 'A',
                replace: false,
                templateUrl: 'assets/public/directives/news/article/rate.html',
                link: function (s) {
                }
            };
        }]);
    app.directive('smnHideInPage', ['page_manager', function (page_manager) {
        return {
            restrict: 'A',
            replace: true,
            link: function (s, elem) {
                var id = page_manager.page_id();
                if (id !== undefined) {
                    elem.hide();
                }
            }
        };
    }]);
    app.directive('smnModalXStatus', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                link: function (scope, elem) {
                    scope.modal_status = {open: undefined};
                    scope.modal_status.open = function () {
                        elem.modal('show');
                    };
                    scope.$broadcast('status', function () {
                        return 1;
                    });
                    scope.$on('$routeChangeStart', function () {
                        elem.modal('hide');
                    });
                }
            };
        }]);
    app.directive('smnReadArticle', ['news_manager',
        function (news_manager) {
            return {
                restrict: 'A',
                replace: true,
                scope: {axid: '=', xtype: '@'},
                link: function (scope, elem) {
                    function run() {
                        if (news_manager.is_read(scope.axid) === true) {
                            if (scope.xtype === '1') {
                                elem.addClass('fa-check');
                                elem.removeClass('fa-chevron-right');
                            } else if (scope.xtype === '2') {
                                elem.addClass('fa-plus');
                                elem.removeClass('fa-minus');
                            }

                        } else {
                            if (scope.xtype === '1') {
                                elem.addClass('fa-chevron-right');
                            } else if (scope.xtype === '2') {
                                elem.addClass('fa-minus');
                            }
                        }
                    }

                    scope.$on('finally', function () {
                        run();
                    });
                    run();
                }
            };
        }]);
    app.directive('smnArticleModalDialog', [
        function () {
            return {
                restrict: 'A',
                replace: false,
                link: function (scope, elem) {
                    scope.$on('$routeChangeStart', function () {
                        elem.modal('hide');
                    });
                    elem.on('hidden.bs.modal', function () {
                        scope.$broadcast('done', function () {
                            return 0;
                        });
                    });
                }
            };
        }]);
    app.directive('smnArticlePage', ['$rootScope',
        function () {
            return {
                restrict: 'A',
                replace: true,
                scope: {art: '=', xbreaks: '=', xtrim: '=', xflag: '='},
                templateUrl: 'assets/public/directives/news/article/article.html',
                link: function (scope) {
                    scope.xflag = true;
                    scope.$on('$destroy', function () {
                        scope.$broadcast('done', function () {
                            return 1;
                        });
                    });
                }
            };
        }]);
    app.directive('smnNewsCarousel', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                link: function (scope, elem) {
                    angular.element('.swipe-area').addClass('smn-c');
                    elem.carousel({interval: 4000, cycle: true});
                    scope.activeLi = function (index) {
                        return index === 0 ? 'active' : '';
                    };
                    scope.activeIt = function (index) {
                        return index === 0 ? 'item active' : 'item';
                    };
                    scope.prev = function () {
                        elem.carousel('prev');
                    };
                    scope.next = function () {
                        elem.carousel('next');
                    };
                    scope.$on('$destroy', function () {
                        angular.element('.swipe-area').removeClass('smn-c');
                    });
                }
            };
        }]);
    app.directive('smnArticleRatePanel', ['jQuery', 'hotWire', 'news_manager',
        function (jQuery, hotWire, nm) {
            return {
                restrict: 'A',
                replace: true,
                scope: {xaid: '='},
                link: function (scope, elem) {
                    function set_rating(ob, nx) {
                        var allStars = 5,
                            numStars = ob !== undefined ? jQuery(ob).nextUntil().length + 1 : nx,
                            rest = allStars - numStars,
                            i,
                            current,
                            j,
                            f = false,
                            k = 0,
                            es = jQuery('#1');

                        if (numStars === 1 && !es.hasClass('rsx')) {
                            f = true;
                        }
                        for (j = 1; j <= numStars; j++) {
                            jQuery('#' + j).addClass('rsx');
                        }
                        for (i = 5; i > (allStars - rest); i--) {
                            current = jQuery('#' + i);
                            if (current.hasClass('rsx')) {
                                k = (k + 1);
                                current.removeClass('rsx');
                            }
                        }
                        if (numStars === 1 && es.hasClass('rsx') && !f && k === 0) {
                            jQuery('#1').removeClass('rsx');
                            numStars = 0;
                        }

                        hotWire.event.rating = numStars;
                        if (ob !== undefined) {
                            nm.save_rate(scope.xaid, numStars);
                        }

                    }

                    function auto_rate() {
                        var r = nm.get_rate(hotWire.event.article_id), s = elem.attr('id'), i = parseInt(s, 0);
                        if (i <= r) {
                            set_rating(undefined, r);
                        }
                    }

                    auto_rate();
                    elem.on("click", function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        set_rating(this);
                    });
                    scope.$on('done', function () {
                        elem.removeClass('rsx');
                    });
                    scope.$on('reads', function () {
                        auto_rate();
                    });
                }
            };
        }]);
    app.directive('smnArticleTimer', ['timer_service', '$interval', 'hotWire', '$rootScope',
        function (ts, $interval, hotWire, $rootScope) {
            return {
                restrict: 'A',
                replace: false,
                scope: {xflag: '='},
                link: function (scope, elm) {
                    function start() {
                        scope.startTime = new Date().getTime();
                        hotWire.event.timestamp = scope.startTime;
                        scope.timer = $interval(function () {
                            scope.seconds = (new Date().getTime() - scope.startTime);
                            elm.text(ts.time(scope.seconds));
                            hotWire.event.time_spent = scope.seconds;
                        }, 1000);
                    }

                    function finish() {
                        scope.xflag = false;
                        if (angular.isDefined(scope.timer)) {
                            $interval.cancel(scope.timer);
                        }
                        elm.text('00:00:00');
                        $rootScope.$broadcast('finally', function () {
                            return 1;
                        });
                    }

                    scope.$watch('xflag', function (nval) {
                        if (nval === true) {
                            start();
                        }
                    });
                    scope.$on('done', function () {
                        finish();
                    });
                }
            };
        }]);
    app.directive('smnArticlePrefSize', ['preferences',
        function (pref) {
            return {
                restrict: 'A',
                replace: true,
                scope: {},
                link: function (scope, elem) {
                    function run() {
                        var size = pref.xget_pref();
                        if (size !== undefined && size.fsize !== undefined) {
                            elem.css('font-size', pref.font_pref(size.fsize));
                        }
                    }

                    run();
                    scope.$on('size', function () {
                        run();
                    });
                    scope.$on('user_prefs', function () {
                        run();
                    });
                }
            };
        }]);
    app.directive('smnArticleContentBig', ['preferences', '$rootScope',
        function (pref, $rootScope) {
            return {
                restrict: 'A',
                replace: true,
                scope: {},
                link: function (scope, elem) {
                    scope.c = 0;
                    function change() {
                        var prefs = pref.xget_pref(), t;
                        if (prefs !== undefined && prefs.fsize !== undefined && prefs.fsize < 8) {
                            t = prefs.fsize;
                            t = (t + 1);
                            prefs.fsize = t;
                            pref.xset_pref(prefs);
                            $rootScope.$broadcast('size', function () {
                                return 1;
                            });
                            scope.c = scope.c + 1;
                        }
                    }

                    elem.on('click', function () {
                        change();
                    });
                    scope.$watch('c', function () {
                    });
                    scope.$on('user_prefs', function () {
                        change();
                    });
                }
            };
        }]);
    app.directive('smnArticleContentSmall', ['$rootScope', 'preferences',
        function ($rootScope, pref) {
            return {
                restrict: 'A',
                replace: false,
                scope: {},
                link: function (scope, elem, attrs) {
                    scope.c = 0;
                    function change() {
                        var prefs = pref.xget_pref(), t;
                        if (prefs !== undefined && prefs.fsize !== undefined && prefs.fsize > 1) {
                            t = prefs.fsize;
                            t = (t - 1);
                            prefs.fsize = t;
                            pref.xset_pref(prefs);
                            $rootScope.$broadcast('size', function () {
                                return 1;
                            });
                            scope.c = scope.c + 1;
                        }
                    }

                    elem.on('click', function () {
                        change();
                    });
                    scope.$watch('c', function () {
                    });
                    scope.$on('user_prefs', function () {
                        change();
                    });
                }
            };
        }]);
    app.directive('smnArticleBgChanger', ['preferences', '$rootScope',
        function (pref, $rootScope) {
            return {
                restrict: 'A',
                replace: true,
                link: function (scope, elem) {
                    function change() {
                        var prefs = pref.xget_pref();
                        if (prefs !== undefined && prefs.bcolor !== undefined) {
                            if ((prefs.bcolor === 1)) {
                                prefs.bcolor = 2;
                            } else if (prefs.bcolor === 2) {
                                prefs.bcolor = 3;
                            } else if (prefs.bcolor === 3) {
                                prefs.bcolor = 4;
                            } else if (prefs.bcolor === 4) {
                                prefs.bcolor = 5;
                            } else if (prefs.bcolor === 5) {
                                prefs.bcolor = 1;
                            }
                            pref.xset_pref(prefs);
                            $rootScope.$broadcast('color', function () {
                                return 1;
                            });
                        }
                    }

                    elem.on('click', function () {
                        change();
                    });
                    scope.$on('user_prefs', function () {
                        change();
                    });
                }
            };
        }]);
    app.directive('smnArticlePrefColor', ['preferences', 'Math',
        function (pref) {
            return {
                restrict: 'A',
                replace: true,
                scope: {ctype: '='},
                link: function (scope, elem) {
                    function removeAll() {
                        elem.removeClass('xa-a xa-b xa-c xa-d xa-e');
                    }

                    function run() {
                        var prefs = pref.xget_pref();
                        if (prefs !== undefined && prefs.bcolor !== undefined) {
                            if ((prefs.bcolor === 1)) {
                                removeAll();
                                elem.addClass('xa-a');
                            } else if (prefs.bcolor === 2) {
                                removeAll();
                                elem.addClass('xa-b');
                            } else if (prefs.bcolor === 3) {
                                removeAll();
                                elem.addClass('xa-c');
                            } else if (prefs.bcolor === 4) {
                                removeAll();
                                elem.addClass('xa-d');
                            } else if (prefs.bcolor === 5) {
                                removeAll();
                                elem.addClass('xa-e');
                            }
                        }
                    }

                    run();
                    scope.$on('color', function () {
                        run();
                    });
                    scope.$on('user_prefs', function () {
                        run();
                    });
                }
            };
        }]);
    app.directive('smnShareArticle', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/article/share.html'
            };
        }]);
    app.directive('smnTwitterShare', ['page_manager', 'hotWire', function (pm, hw) {
        return {
            restrict: 'A',
            replace: true,
            template: '<i class="fa fa-twitter-square fa-3x twitter smn-time pointer" data-smn-scale-that data-text="Smart media news"></i>',
            link: function (s, elem) {
                elem.sharrre({
                    share: {
                        twitter: true
                    },
                    enableHover: false,
                    enableCounter: false,
                    template: '<div></div>',
                    click: function (api, options) {
                        options.url = pm.share_link(hw.event.article_id);
                        api.simulateClick();
                        api.openPopup('twitter');
                    }
                });
            }
        };
    }]);
    app.directive('smnLinkedInShare', ['page_manager', 'hotWire', function (pm, hw) {
        return {
            restrict: 'A',
            replace: true,
            template: '<i class="fa fa-linkedin-square fa-3x linkedin smn-time pointer" data-smn-scale-that data-text="Smart media news"></i>',
            link: function (s, elem) {
                elem.sharrre({
                    share: {
                        linkedin: true
                    },
                    enableHover: false,
                    enableCounter: false,
                    template: '<div></div>',
                    click: function (api, options) {
                        options.url = pm.share_link(hw.event.article_id);
                        api.simulateClick();
                        api.openPopup('linkedin');
                    }
                });
            }
        };
    }]);
    app.directive('smnGooglePlusShare', ['page_manager', 'hotWire', function (pm, hw) {
        return {
            restrict: 'A',
            replace: true,
            template: '<i class="fa fa-google-plus-square fa-3x googleplus smn-time pointer" data-smn-scale-that data-text="Smart media news"></i>',
            link: function (s, elem) {
                elem.sharrre({
                    share: {
                        googlePlus: true
                    },
                    enableHover: false,
                    enableCounter: false,
                    template: '<div></div>',
                    click: function (api, options) {
                        options.buttons.googlePlus.url = pm.share_link(hw.event.article_id);
                        api.simulateClick();
                        api.openPopup('googlePlus');
                    }
                });
            }
        };
    }]);
    app.directive('smnFacebookShare', ['page_manager', 'hotWire', function (pm, hw) {
        return {
            restrict: 'A',
            replace: true,
            template: '<i class="fa fa-facebook-square fa-3x facebook smn-time pointer" data-smn-scale-that data-text="Smart media news""></i>',
            link: function (s, elem) {
                elem.sharrre({
                    share: {
                        facebook: true
                    },
                    enableHover: false,
                    enableCounter: false,
                    template: '<div></div>',
                    click: function (api, options) {
                        options.buttons.facebook.url = pm.share_link(hw.event.article_id);
                        api.simulateClick();
                        api.openPopup('facebook');
                    }
                });
            }
        };
    }]);
    app.directive('smnArticlePanel', [
        function () {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/news/article/panel.html',
                scope: {xflag: '=', xaid: '=', art: '='},
                link: function (scope) {
                    function reset() {
                        scope.one = scope.two = scope.three = true;
                        scope.four = false;
                    }

                    reset();
                    scope.$on('done', function () {
                        reset();
                    });
                    scope.$on('reads', function () {
                        scope.four = true;
                    });
                    scope.$on('map_ready', function (e, s) {
                        var a = s();
                        if (a === 0) {
                            scope.four = true;
                        }
                    });
                    scope.$on('get_location', function () {
                        scope.four = false;
                    });
                }
            };
        }]);
    app.directive('smnGoogleMap', ['$rootScope', function ($rootScope) {
        return {
            restrict: 'A',
            replace: true,
            templateUrl: 'assets/public/directives/news/article/map.html',
            link: function (scope) {
                scope.turn_geo = function () {
                    $rootScope.$broadcast('manual_geo', function () {
                        return 1;
                    });
                };
            }
        };
    }]);
    app.directive('smnMapPart', ['$rootScope', 'preferences', 'hotWire', function ($rootScope, prefs, hw) {
        return {
            restrict: 'A',
            replace: true,
            templateUrl: 'assets/public/directives/news/parts/map_part.html',
            scope: { articles: '=', showart: '='},
            controller: function ($scope) {
                $scope.map_ctrl = {};
                $scope.markers = [];
                $scope.set_marker = function (m) {
                    m.i = 'assets/img/site/default/home.png';
                };
                angular.forEach($scope.articles, function (v) {
                    if (v.geo !== undefined && v.geo.length > 0) {
                        angular.forEach(v.geo, function (vg) {
                            angular.forEach(vg, function (vl) {
                                $scope.markers.push(
                                    {   cl: function () {
                                        $scope.showart(this.model.options.a, true);
                                    },
                                        svl: false,
                                        url: 'assets/public/directives/news/parts/map_label.html',
                                        c: {latitude: vl[0], longitude: vl[1]},
                                        events: $scope.events,
                                        options: {
                                            title: v.src.title + ':\n' + v.src.leadText,
                                            a: v
                                        }
                                    }
                                );
                            });
                        });
                    }
                });
                function user_loc() {
                    var loc = prefs.get_geo();
                    if (loc !== undefined) {
                        $scope.markers.push(
                            {i: 'assets/img/site/default/home.png',
                                c: {latitude: loc.latitude, longitude: loc.longitude },
                                sw: false, title: 'ME',
                                anchor: {x: 10, y: 10}}
                        );
                    }
                }

                user_loc();
                $scope.center = { latitude: 13.440227,
                    longitude: 22.450947};
                $scope.zoom = 3;
                $scope.icon = '';
                $scope.bounds = { northeast: {latitude: 2.5404, longitude: -3.0928}, southwest: {latitude: 4.8863, longitude: -15.0264}};
                $scope.pan = false;
                $scope.anchor = {x: 10, y: 10};
                $scope.options = {
                    disableDefaultUI: true,
                    draggable: true,
                    minZoom: 3
                    /*zoomControl: false,
                     scrollwheel: false,
                     scaleControl: false,
                     rotateControl: false,
                     panControl: false,
                     mapTypeControl: false,
                     overviewMapControl: false,
                     backgroundColor: 'red'*/
                };
                $scope.$on('delete_geo', function () {
                    $scope.markers.pop();
                });
                $scope.$on('location_on', function () {
                    user_loc();
                });
            }
        };
    }]);
    app.directive('smnNewsMapView', [function () {
        return {
            restrict: 'A',
            replace: true,
            templateUrl: 'assets/public/directives/news/view/world.html',
            scope: {articles: '=', xopen: '='}
        };
    }]);
    app.directive('smnSetSearch', ['$timeout', function ($timeout) {
        return {
            restrict: 'A',
            replace: false,
            link: function (scope, elem) {
                scope.$on('filter_m', function () {
                    $timeout(function () {
                        elem.focus();
                    }, 2000);
                });
            }
        };
    }]);
}());