(function () {
    'use strict';
    var app = angular.module('smnNg');
    app.directive('smnAuth', ['$timeout', 'animation', 'jQuery', 'notifier', 'user_manager', 'preferences', 'menu', '$rootScope', 'page_manager',
        function ($timeout, animation, jQuery, notifier, user_manager, pref, menu, $rootScope, pm) {
            return {
                restrict: 'A',
                templateUrl: 'assets/public/directives/modal/auth.html',
                replace: true,
                link: function (scope) {
                    scope.auth = {auth: undefined, register: undefined, log_out: undefined, set_location: undefined,
                        show_profile: undefined, identity: undefined, location: undefined};
                    scope.auth.identity = user_manager.identify_user();

                    function start_request(elem) {
                        animation.toggle(elem);
                    }

                    function start_sm_request(elem, size) {
                        animation.toggle(elem, undefined, size);
                    }

                    function end_request(form, elem, addition, error) {
                        if (error === 0) {
                            form.$setPristine();
                            form.email = undefined;
                            form.password = undefined;
                        }
                        animation.toggle(elem, addition);
                    }

                    scope.auth.set_location = function () {

                        var allow = pref.xget_pref();
                        if (allow.location) {
                            user_manager.current_loc().then(function (location) {
                                pref.set_geo(location);
                                scope.$broadcast('location_on', function () {
                                    return 1;
                                });
                            });
                        }
                    };
                    function handle_request(result, form, elem, addition, action) {

                        if (result !== undefined) {
                            scope.auth.set_location();
                            $timeout(function () {
                                jQuery('#auth-modal').modal('hide');
                                end_request(form, elem, addition, 0);
                                $timeout(function () {
                                    if (action === 0) {
                                        notifier.notify('You have successfully signed in', 'success');
                                    } else if (action === 1) {
                                        notifier.notify('You have successfully registered and signed in', 'success');
                                    }
                                    scope.$broadcast('proceed', function () {
                                        return 1;
                                    });
                                }, 500);
                            }, 1500);
                        } else if (result === undefined) {
                            $timeout(function () {
                                end_request(form, elem, 'Try again', 1);
                                form.$valid = true;
                                if (action === 0) {
                                    notifier.notify('Wrong username or password', 'error');
                                } else if (action === 1) {
                                    notifier.notify('This email is already registered, please sing in', 'error');
                                } else if (action === -1) {
                                    notifier.notify('Something went very wrong', 'error');
                                }
                            }, 1500);
                            return 1;
                        }
                    }

                    function declare_identity(result) {
                        if (result !== undefined) {
                            scope.auth.identity = user_manager.identify_user();
                        }
                    }

                    scope.auth.auth = function (form) {
                        if (form !== undefined && form.$valid) {
                            if (form.email.length >= 7 && form.password.length >= 5) {
                                form.$valid = false;
                                start_request('#isub');
                                var user = {email: form.email, password: form.password}
                                user_manager.authenticate_user(user).
                                    then(function (result) {
                                        declare_identity(result);
                                        handle_request(result, form, '#isub', 'Sign In', 0);
                                        if (result !== undefined) {
                                            user.uid = result.uid;
                                            user_manager.get_user_prefs(user).then(function (r) {
                                                if (r !== undefined && r.prefs !== undefined && Object.keys(r.prefs).length >= 8) {
                                                    pref.xset_pref(r.prefs);
                                                    $rootScope.$broadcast('user_prefs', function () {
                                                        return 1;
                                                    });
                                                }
                                            });
                                        }
                                    }, function () {
                                        handle_request(undefined, form, '#isub', 'Sign In', -1);
                                    }
                                        );
                            }
                        }
                    };
                    scope.auth.register = function (form) {
                        if (form !== undefined && form.$valid) {
                            if (form.email.length >= 7 && form.password.length >= 5) {
                                form.$valid = false;
                                start_request('#usub');
                                user_manager.register_user({email: form.email, password: form.password}).
                                    then(function (result) {
                                        declare_identity(result);
                                        handle_request(result, form, '#usub', 'Sign Up', 1);
                                    }, function () {
                                        handle_request(undefined, form, '#usub', 'Sign Up', -1);
                                    }
                                        );
                            }
                        }
                    };
                    scope.auth.log_out = function (action) {
                        user_manager.set_user_prefs(scope.auth.identity, pref.xget_pref());
                        var wait = 1500;
                        if (action === 1) {
                            start_sm_request('#alogout', 1);
                        } else {
                            animation.anima_i('#lflo', '', 'no-borders fa-spin fa-spinner');
                        }
                        user_manager.logout_user().
                            then(function (result) {
                                if (result === true) {
                                    if (pm.page_id() !== undefined) {
                                        pm.navigate('news');
                                    }
                                    $timeout(function () {
                                        scope.auth.identity = undefined;
                                        notifier.notify('You have successfully logged out', 'warning');
                                    }, wait);
                                }
                            }
                                );
                    };
                    scope.auth.show_profile = function () {

                        notifier.notify('Getting profile...');
                        user_manager.get_user_profile(scope.auth.identity).then(
                            function (response) {
                                scope.profile = response.profile;
                                scope.auth.location = pref.get_geo();
                                var wait = 100;
                                if (jQuery('.smnlfnav').hasClass('nav-open')) {
                                    scope.left_btn.toggle_left_nav();
                                    menu.toggleLeft();
                                    wait = 700;
                                }
                                $timeout(function () {
                                    jQuery('#profile-modal').modal('show');
                                    jQuery('#pfname').focus();
                                }, wait);
                            }
                        );
                    };
                    (function proceed() {
                        $rootScope.$broadcast('proceed', function () {
                            return 1;
                        });
                    }());
                }
            };
        }]);

    app.directive('smnFlipper', ['jQuery', 'common_service', '$timeout', function (jQuery, common_service, $timeout) {
        return {
            restrict: 'A',
            replace: true,
            scope: { xplace: '='},
            link: function (scope, elem) {
                var formContainer, fid, id;
                if (scope.xplace === 0) {
                    fid = '#formContainer';
                    id = '#umail';
                } else if (scope.xplace === 1) {
                    fid = '#formContainer';
                    id = '#imail';
                } else {
                    fid = '#xformContainer';
                    id = '#pfname';
                }
                elem.click(function (event) {
                    event.preventDefault();
                    formContainer = jQuery(fid);
                    formContainer.toggleClass('flipped');
                    if (!common_service.is_transform3d()) {
                        jQuery('#login').toggle();
                    }
                    $timeout(function () {
                        jQuery(id).focus();
                    }, 1000);
                });
            }
        };
    }]);

    app.directive('smnProfile', ['$timeout', 'animation', 'user_manager', 'constant_service', 'notifier',
        function ($timeout, animation, user_manager, cs, notifier) {
            return {
                restrict: 'A',
                templateUrl: 'assets/public/directives/modal/profile.html',
                replace: true,
                link: function (scope) {


                    scope.pf = {switch_m: undefined, save: undefined, col: {inf: false, edi: true}};

                    scope.days = cs.numbers(1, 31);
                    scope.months = cs.numbers(1, 12);
                    scope.years = cs.numbers(1900, 2000);
                    scope.occ = cs.app_occ();

                    scope.pf.switch_m = function (mode) {

                        if (mode === 1) {
                            scope.pf.col.inf = !scope.pf.col.inf;
                            $timeout(function () {
                                scope.pf.col.edi = !scope.pf.col.edi;
                            }, 300);
                        } else {
                            scope.pf.col.edi = !scope.pf.col.edi;
                            $timeout(function () {
                                scope.pf.col.inf = !scope.pf.col.inf;
                            }, 300);
                        }
                    };
                    function set_bd(p) {
                        return p.day + '/' + p.month + '/' + p.year;
                    }

                    scope.pf.save = function (form, profile) {
                        if (form.$valid) {
                            animation.toggle('#pfsub', undefined, 1);
                            notifier.notify('Updating profile...', 'warning');
                            profile.birthday.bd = set_bd(profile.birthday);
                            user_manager.set_user_profile(scope.auth.identity, profile).
                                then(function (result) {
                                    if (result === 1) {
                                        notifier.notify('Your profile is now updated');
                                        $timeout(function () {
                                            scope.pf.switch_m();
                                            $timeout(function () {
                                                animation.toggle('#pfsub', 'SAVE');
                                            }, 100);
                                        }, 300);
                                    }
                                });
                        }
                    };
                }
            };
        }]);
    app.directive('smnStatusModal', [ 'modal_manager',
        function (mm) {
            return {
                restrict: 'A',
                replace: true,
                templateUrl: 'assets/public/directives/modal/status.html',
                link: function (scope) {
                    scope.yes = function () {
                        mm.cl_stat();
                    };
                }
            };
        }]);
}());