(function () {
    'use strict';
    var app = angular.module('smnNg');
    app.factory('request_manager', ['$http', '$q', function ($http, $q) {
        function async_request(http) {
            var defer = $q.defer();
            $http({
                method: http.method,
                url: http.xurl || ('http://smartnews.idi.ntnu.no/' + http.url),
                data: http.data

            }).
                success(function (data, status) {
                    /*console.log(http.info + ' success status: ' + status);
                    console.log(data);*/
                    defer.resolve(data);

                }).
                error(function (data, status) {
                    /*console.log(http.info + ' error status: ' + status);
                    console.log(data);*/
                    defer.reject(data);
                });
            return defer.promise;
        }

        return {
            async_request: async_request
        };
    }]);

    app.factory('store_manager', ['localStorage', function (localStorage) {

        function add_raw_store(key, item) {
            if (key && item) {
                localStorage.setItem(key, item);
            }
        }

        function get_raw_store(key) {
            if (key) {
                var raw = localStorage.getItem(key);
                if (raw === null || undefined) {
                    return undefined;
                }
                return raw;
            }
        }

        function add_to_store(key, item) {
            if (key && item) {
                item = JSON.stringify(item);
                try {
                    localStorage.setItem(key, item);
                } catch (error) {
                    console.log('..');
                } finally {
                    return true;
                }
                return true;
            }
            return false;
        }

        function get_from_store(key) {
            if (key) {
                var raw = localStorage.getItem(key);
                if (raw !== null) {
                    raw = JSON.parse(raw);
                    return raw;
                }
                return undefined;
            }
        }

        function remove_from_store(key) {
            if (key) {
                localStorage.removeItem(key);
            }
        }

        return {
            add_to_store: add_to_store,
            get_from_store: get_from_store,
            remove_from_store: remove_from_store,
            add_raw_store: add_raw_store,
            get_raw_store: get_raw_store
        };
    }]);
    app.factory('preferences', ['store_manager', 'jQuery', function (store_manager, jQuery) {
        function set_geo(loc) {
            store_manager.add_to_store('geo', loc);
        }

        function get_geo() {
            return store_manager.get_from_store('geo');
        }

        function delete_geo(key) {
            store_manager.remove_from_store(key);
        }

        function xdelete() {
            store_manager.remove_from_store('x_pref');
        }

        function xget_pref() {
            return store_manager.get_from_store('x_pref');
        }

        function xset_pref(pref) {
            var base = xget_pref();
            if (base !== undefined) {
                base = jQuery.extend({}, base, pref);
                store_manager.add_to_store('x_pref', base);
            } else {
                store_manager.add_to_store('x_pref', pref);
            }

        }

        function font_pref(size) {
            var o;
            switch (size) {
                case 1:
                    o = '0.8';
                    break;
                case 2:
                    o = '0.9';
                    break;
                case 3:
                    o = '1';
                    break;
                case 4:
                    o = '1.1';
                    break;
                case 5:
                    o = '1.2';
                    break;
                case 6:
                    o = '1.3';
                    break;
                case 7:
                    o = '1.4';
                    break;
                case 8:
                    o = '1.5';
                    break;
                case 9:
                    o = '1.6';
                    break;
                case 10:
                    o = '1.7';
                    break;
                case 11:
                    o = '1.8';
                    break;
                case 12:
                    o = '1.9';
                    break;
                case 13:
                    o = '2';
                    break;
                default:
                    o = '1';
                    break;
            }
            return o + 'em';
        }

        function bg_pref(p) {
            var a = [];
            switch (p) {
                case 1:
                    a.push({bc: 'rgb(253,233,235)', fc: '#333333'});
                    break;
                case 2:
                    a.push({bc: 'rgb(35,35,35)', fc: 'rgb(134, 185, 51)'});
                    break;
                case 3:
                    a.push({bc: 'rgb(241,241,241)', fc: '#333333'});
                    break;
                default:
                    a.push({bc: 'rgb(35,35,35)', fc: 'rgb(241,241,241)'});
                    break;
            }
            return a;
        }

        return {
            set_geo: set_geo,
            get_geo: get_geo,
            delete_geo: delete_geo,
            xset_pref: xset_pref,
            xget_pref: xget_pref,
            font_pref: font_pref,
            bg_pref: bg_pref
        };
    }]);
    app.factory('animation', ['jQuery', function (jQuery) {

        function toggle(elem, addition, size) {
            size = size || 2;
            addition = addition || '<i class="fa fa-' + size + 'x fa-refresh fa-spin"></i>';
            jQuery(elem).html('').html(addition);
        }

        function anima_i(elem, remove, add) {
            jQuery(elem).removeClass(remove).addClass(add);
        }

        return {
            toggle: toggle,
            anima_i: anima_i
        };
    }]);

    app.factory('menu', ['jQuery', function (jQuery) {
        var ctr = jQuery('#main-content'),
            lfBtn = jQuery('#left-menu-toggle'),
            lfSwipe = jQuery('.left-swipe-area'),
            rtBtn = jQuery('#right-menu-toggle'),
            rtSwipe = jQuery('.right-swipe-area'),
            toggleLeft = function () {
                jQuery('.lfnav').toggleClass('nav-left nav-open');
                lfBtn.toggleClass('mc-left-push');
                lfSwipe.toggleClass('mc-left-push');
                rtBtn.toggleClass('mc-left-push');
                rtSwipe.toggleClass('mc-left-push');
                ctr.toggleClass('mc-left-push');
            },
            toggleRight = function () {
                jQuery('.rtnav').toggleClass('nav-right nav-open');
                rtBtn.toggleClass('mc-right-push');
                rtSwipe.toggleClass('mc-right-push');
                lfBtn.toggleClass('mc-right-push');
                lfSwipe.toggleClass('mc-right-push');
                ctr.toggleClass('mc-right-push');
            };
        return {
            toggleLeft: toggleLeft,
            toggleRight: toggleRight
        };
    }]);
    app.factory('notifier', ['toastr',
        function (toastr) {

            (function setToasterOptions() {
                toastr.options.closeButton = true;
            }());

            function notify(message, type) {
                if (type === 'success') {
                    toastr.success(message);
                } else if (type === 'warning') {
                    toastr.warning(message);
                } else if (type === 'error') {
                    toastr.error(message);
                } else {
                    toastr.info(message);
                }
            }

            return {
                notify: notify
            };
        }]);

    app.factory('common_service', ['Math', 'jQuery', 'Modernizr', 'navigator', function (Math, jQuery, Modernizr, navigator) {

        var random_number = function (limit) {
                return Math.floor((Math.random() * limit));
            },
            is_touch_device = function () {
                return Modernizr.touch === true;
            },
            is_mobile_device = function () {
                return jQuery.browser.mobile === true;
            },
            is_device = {
                Android: function () {
                    return navigator.userAgent.match(/Android/i);
                },
                BlackBerry: function () {
                    return navigator.userAgent.match(/BlackBerry/i);
                },
                iOS: function () {
                    return navigator.userAgent.match(/iPhone|iPad|iPod/i);
                },
                iPhone: function () {
                    return navigator.userAgent.match(/iPhone/i);
                },
                iPad: function () {
                    return navigator.userAgent.match(/iPad/i);
                },
                Opera: function () {
                    return navigator.userAgent.match(/Opera Mini/i);
                },
                Windows: function () {
                    return navigator.userAgent.match(/IEMobile/i);
                },
                any: function () {
                    return (is_device.Android() || is_device.BlackBerry() || is_device.iOS() || is_device.Opera() || is_device.Windows());
                }
            },
            is_browser = {
                safari: function () {
                    return navigator.userAgent.match(/(iPad|iPhone|iPod touch);.*CPU.*OS 7_\d/i);
                },
                chrome: function () {
                    return navigator.userAgent.indexOf('Chrome') > -1;

                },
                firefox: function () {
                    return navigator.userAgent.indexOf('Firefox') > -1;
                },
                which_browser: function () {
                    is_browser.safari();
                    is_browser.chrome();
                    is_browser.firefox();

                }
            };

        function is_transform3d() {
            return Modernizr.csstransforms3d;
        }

        function extend(base, addons) {
            return jQuery.extend({}, base, addons);
        }

        function numbers(startRange, endRange) {
            var nums = [], i, option;
            for (i = startRange; i <= endRange; i++) {
                option = {};
                if (i < 10) {
                    option.name = '0' + i.toString();
                    option.value = '0' + i.toString();
                } else {
                    option.name = i;
                    option.value = i;
                }
                nums.push(option);
            }
            return nums;
        }

        function random_logo() {

            var logo = 'assets/img/site/default/', logos;

            if (is_mobile_device()) {
                logo = logo + 'port/';

            } else if (!is_mobile_device()) {
                logo = logo + 'desk/';
            }

            logos = ['black_red', 'black_blue', 'black_green', 'black_orange', 'gold_orange',
                'orange_black', 'white_red', 'white', 'red_white', 'black_pant'];

            return logo + logos[random_number(logos.length - 1)] + '.gif';
        }

        function random_colors() {

            var colors = ['rgb(160,236,111)', 'rgb(255,130,130)',
                'rgb(90,198,255)', 'rgb(28,154,115)', 'rgb(245,230,102)',
                'rgb(253,193,98)', 'rgb(246,141,187)', 'rgb(10,194,210)',
                'rgb(123,183,250)', 'rgb(245,222,179)', 'rgb(183,183,183)', 'rgb(134, 185, 51)'];

            return colors[random_number(colors.length - 1)];
        }

        function is_smartPhone() {
            return is_mobile_device() && is_touch_device();
        }

        return {
            random_colors: random_colors,
            random_logo: random_logo,
            is_transform3d: is_transform3d,
            extend: extend,
            is_touch_device: is_touch_device,
            is_mobile_device: is_mobile_device,
            random_number: random_number,
            is_device: is_device,
            is_browser: is_browser,
            numbers: numbers,
            is_smartPhone: is_smartPhone
        };
    }]);

    app.factory('constant_service', [
        function () {
            function numbers(startRange, endRange) {
                var nums = [], i, option;
                for (i = startRange; i <= endRange; i++) {
                    option = {};
                    if (i < 10) {
                        option.name = '0' + i.toString();
                        option.value = '0' + i.toString();
                    } else {
                        option.name = i;
                        option.value = i;
                    }
                    nums.push(option);
                }
                return nums;
            }

            function app_occ() {
                return [
                    {name: 'Other', value: 'Other'},
                    {name: 'Student', value: 'Student'},
                    {name: 'Unemployed', value: 'Unemployed'},
                    {name: 'Accounting & Auditing', value: 'Accounting & Auditing'},
                    {name: 'Administrative & Support', value: 'Administrative & Support'},
                    {name: 'Advertising, Marketing', value: 'Advertising, Marketing'},
                    {name: 'Arts, Entertainment & Media', value: 'Arts, Entertainment & Media'},
                    {name: 'Banking', value: 'Banking'},
                    {name: 'Consulting Services', value: 'Consulting Services'},
                    {name: 'Design', value: 'Design'},
                    {name: 'Engineering', value: 'Engineering'},
                    {name: 'Finance & Economics', value: 'Finance & Economics'},
                    {name: 'Human Resources', value: 'Human Resources'},
                    {name: 'Industry', value: 'Industry'},
                    {name: 'Information Technology', value: 'Information Technology'},
                    {name: 'Law Enforcement', value: 'Law Enforcement'},
                    {name: 'Manufacturing', value: 'Manufacturing'},
                    {name: 'Restaurant & Food Service', value: 'Restaurant & Food Service'},
                    {name: 'Personal Care', value: 'Personal Care'},
                    {name: 'Public Relations Jobs', value: 'Public Relations Jobs'},
                    {name: 'Sales', value: 'Sales'},
                    {name: 'Science & Research', value: 'Science & Research'},
                    {name: 'Security', value: 'Security'},
                    {name: 'Technician', value: 'Technician'},
                    {name: 'Warehousing & Distribution', value: 'Warehousing & Distribution'}
                ];
            }

            return {
                app_occ: app_occ,
                numbers: numbers
            };
        }]);
    app.factory('timer_service', [function () {
        function time(milliseconds) {
            var oneHour = 3600000,
                oneMinute = 60000,
                oneSecond = 1000,
                seconds = 0,
                minutes = 0,
                hours = 0,
                result;

            if (milliseconds >= oneHour) {
                hours = Math.floor(milliseconds / oneHour);
            }

            milliseconds = hours > 0 ? (milliseconds - hours * oneHour) : milliseconds;

            if (milliseconds >= oneMinute) {
                minutes = Math.floor(milliseconds / oneMinute);
            }

            milliseconds = minutes > 0 ? (milliseconds - minutes * oneMinute) : milliseconds;

            if (milliseconds >= oneSecond) {
                seconds = Math.floor(milliseconds / oneSecond);
            }

            milliseconds = seconds > 0 ? (milliseconds - seconds * oneSecond) : milliseconds;

            if (hours > 0) {
                result = (hours > 9 ? hours : "0" + hours) + ":";
            } else {
                result = "00:";
            }

            if (minutes > 0) {
                result += (minutes > 9 ? minutes : "0" + minutes) + ":";
            } else {
                result += "00:";
            }

            if (seconds > 0) {
                result += (seconds > 9 ? seconds : "0" + seconds);
            } else {
                result += "00";
            }
            return result;
        }

        return {
            time: time
        };
    }]);
    app.factory('page_manager', ['$anchorScroll', '$routeParams', '$location', '$window',
        function ($anchorScroll, $routeParams, $location, $window) {

            function new_page(page) {
                var url = $location.absUrl() + '/' + page;
                $window.open(url);
            }

            function anchor() {
                $anchorScroll();
            }

            function page_id() {
                return $routeParams.id;
            }

            function navigate(url) {
                $location.path(url);
            }

            function share_link(id) {
                if (page_id() !== undefined) {
                    return $location.absUrl();
                } else {
                    return $location.absUrl() + '/' + (id !== undefined ? id : '');
                }
            }

            return {
                anchor: anchor,
                page_id: page_id,
                navigate: navigate,
                share_link: share_link,
                new_page: new_page
            };
        }]);
    app.factory('modal_manager', ['jQuery', 'menu', '$timeout', function (jQuery, menu, $timeout) {
        function menu_op(scope) {
            if (jQuery('.smnrtnav, .smnlfnav').hasClass('nav-open')) {
                if (jQuery('.smnrtnav').hasClass('nav-open')) {
                    scope.right_btn.toggle_right_nav();
                    menu.toggleRight();
                }
                if (jQuery('.smnlfnav').hasClass('nav-open')) {
                    scope.left_btn.toggle_left_nav();
                    menu.toggleLeft();
                }
                return true;
            }
            return false;
        }

        function op(elm, scope) {
            var t = 0;
            if (menu_op(scope)) {
                t = 1000;
            }
            $timeout(function () {
                jQuery(elm).modal('show');
            }, t);
        }

        function op_src(scope) {
            op('#search-modal', scope);
        }

        function op_art(scope) {
            op('#xart-modal', scope);
        }

        function op_sta(scope) {
            op('#status-modal', scope);
        }

        function cl_stat() {
            jQuery('#status-modal').modal('hide');
            $timeout(function () {
                jQuery('#auth-modal').modal('show');
                $timeout(function () {
                    jQuery('#imail').focus();
                }, 500);
            }, 200);
        }

        function op_au(scope) {
            op('#auth-modal', scope);
            $timeout(function () {
                jQuery('#imail').focus();
            }, 500);
        }

        function open_profile() {

        }

        return {
            op_art: op_art,
            menu_op: menu_op,
            op_sta: op_sta,
            cl_stat: cl_stat,
            op_au: op_au,
            op_src: op_src
        };
    }]);
}());