(function () {
    'use strict';
    var app = angular.module('smnNg');
    app.factory('event_service', ['request_manager', function (request_manager) {
        function post_events(events) {
            var http = {method: 'POST', url: 'events', data: events, info: 'post_events'};
            return request_manager.async_request(http);
        }

        return {
            post_events: post_events
        };
    }]);

    app.factory('event_manager', ['$q', 'event_service', 'identity_manager', 'store_manager',
        function ($q, event_service, identity_manager, store_manager) {
            function save_rating(r) {
                store_manager.add_to_store('x_rate', r);
            }

            function get_rating_store() {
                return store_manager.get_from_store('x_rate');
            }

            function clean_rating_store() {
                store_manager.remove_from_store('x_rate');
            }

            function post_events(user, events) {
                var defer = $q.defer(), identity;
                if (user !== undefined && events.length > 0) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        identity.events = events;
                        event_service.post_events(identity).
                            then(
                            function (response) {
                                defer.resolve(response);
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function register_ratings(user, events) {
                var defer = $q.defer();
                post_events(user, events).
                    then(function (response) {
                        if (response.error === 0 && response.nevents === events.length) {
                            defer.resolve(true);
                        } else {
                            defer.resolve(false);
                        }
                    }, function () {
                        defer.resolve(false);
                    });
                return defer.promise;
            }

            function add_all_ratings(user) {
                var ratings = get_rating_store();
                register_ratings(user, ratings).then(function (result) {
                    if (result) {
                        clean_rating_store();
                    }
                    return result;
                });
            }

            function add_rating_store(user, rating) {
                var ratings = get_rating_store(), s = true;
                if (ratings === undefined) {
                    ratings = [];
                    ratings.push(rating);
                    save_rating(ratings);
                } else {
                    angular.forEach(ratings, function (v) {
                        if (v.article_id === rating.article_id) {
                            if (v.rating !== rating.rating) {
                                v.rating = rating.rating;
                                save_rating(ratings);
                            }
                            s = false;
                        }
                    });
                    if (s) {
                        ratings.push(rating);
                        save_rating(ratings);
                    }
                    if (ratings.length >= 20) {
                        add_all_ratings(user);
                    }
                }
            }

            function add_rating(user, rating) {
                var defer = $q.defer(), identity;
                if (user !== undefined && rating !== undefined) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        add_rating_store(user, rating);
                        defer.resolve(true);
                    }
                } else {
                    defer.resolve(false);
                }
                return defer.promise;
            }

            return {
                add_rating: add_rating
            };
        }]);
}());