(function () {
    'use strict';
    var app = angular.module('smnNg');
    app.factory('news_service', ['request_manager', function (request_manager) {

        function get_news(size) {
            var http = {method: 'GET', url: 'search?size=' + (size || ''), data: undefined, info: 'get_news'};
            return request_manager.async_request(http);
        }

        function get_paged_news(page) {
            var http = {method: 'GET', url: 'articles' + (page || ''), data: undefined, info: 'get_paged_news'};
            return request_manager.async_request(http);
        }

        function get_article(id) {
            var http = {method: 'GET', url: 'id/' + (id || ''), data: undefined, info: 'get_article'};
            return request_manager.async_request(http);
        }
        function get_reco() {
            var http = {method: 'GET', url: 'userrec', data: undefined, info: 'get_RECO'};
            return request_manager.async_request(http);
        }

        return {
            get_news: get_news,
            get_paged_news: get_paged_news,
            get_article: get_article,
            get_reco: get_reco
        };
    }]);
    app.factory('news_manager', ['$q', 'news_service', 'store_manager', 'common_service',
        function ($q, news_service, store_manager, cs) {
            function fetch_page(page) {
                var defer = $q.defer();
                news_service.get_paged_news(page).then(
                    function (response) {
                        response.next = response.paging.next.substring(response.paging.next.indexOf('?'));
                        if (response.paging.prev !== undefined) {
                            response.prev = response.paging.prev.substring(response.paging.prev.indexOf('?'));
                        }
                        defer.resolve(response);
                    },
                    function () {
                        defer.resolve(undefined);
                    }
                );
                return defer.promise;
            }

            function fetch_reco() {
                var defer = $q.defer();
                news_service.get_reco().then(
                    function (response) {
                        defer.resolve(response);
                    },
                    function () {
                        defer.resolve(undefined);
                    }
                );
                return defer.promise;
            }

            function clear_repo() {
                store_manager.remove_from_store('r_nx');
            }

            function save(key, news) {
                store_manager.add_to_store(key, news);
            }

            function get(key) {
                return store_manager.get_from_store(key);
            }

            function count(key) {
                var articles = store_manager.get_from_store(key);
                return articles === undefined ? 0 : articles.length;
            }

            function count_news() {
                return count('r_nx');
            }

            function save_repo(news) {
                save('r_nx', news);
            }

            function get_repo() {
                return get('r_nx');
            }

            function empty_db() {
                return get_repo() === undefined;
            }

            function save_read(news) {
                save('r_ax', news);
            }

            function get_read() {
                return get('r_ax');
            }

            function clear_read() {
                store_manager.remove_from_store('r_ax');
            }

            function save_hd(news) {
                save('h_nx', news);
            }

            function get_hd() {
                return get('h_nx');
            }

            function set_time(time) {
                save('rx_t', time);
            }

            function get_time() {
                return get('rx_t');
            }

            function passed_time() {
                var ot = get_time(), nt = new Date().getTime();
                if (ot !== undefined) {
                    ot = new Date(ot);
                    ot = nt - ot;
                    if (ot > (3600000 * 6)) {
                        return true;
                    }
                    return false;
                }
                return false;
            }

            function save_art(xid) {
                var arts = get_read(), ob = {id: xid}, ar = [], f = false;
                if (arts === undefined) {
                    ar.push(ob);
                    save_read(ar);
                } else if (arts !== undefined) {
                    angular.forEach(arts, function (v) {
                        if (v.id === ob.id) {
                            f = true;
                        }
                    });
                    if (!f) {
                        arts.push(ob);
                        save_read(arts);
                    }
                }
            }

            function save_rate(axid, rate) {
                var arts = get_read();
                angular.forEach(arts, function (v) {
                    if (v.id === axid) {
                        v.rate = rate;
                        save_read(arts);
                        return;
                    }

                });
            }

            function get_rate(axid) {
                var arts = get_read(), r = 0;
                angular.forEach(arts, function (v) {
                    if (v.id === axid) {
                        if (v.rate !== undefined) {
                            r = v.rate;
                        }
                    }
                });
                return r;
            }

            function is_read(id) {
                var arts = get_read(), f = false;
                if (arts !== undefined) {
                    angular.forEach(arts, function (v) {
                        if (v.id === id) {
                            f = true;
                        }
                    });
                }
                return f;
            }

            function get_from_repo(size) {
                var articles = get_repo(), result = [], i;
                if (articles !== undefined) {
                    for (i = (size - 50); i < size; i = (i + 1)) {
                        if (articles[i] !== undefined) {
                            result.push(articles[i]);
                        }
                    }
                    return result;
                }
                return undefined;
            }

            function get_news(size) {
                var news = [], imgs = [], defer = $q.defer(), article, geo = [], shape;
                news_service.get_news(size).then(
                    function (response) {
                        angular.forEach(response, function (val) {
                            if (val._source.entities !== undefined && val._source.entities.length > 0) {
                                shape = val._source.entities;
                                angular.forEach(shape, function (v) {
                                    if (v.entity.geoShape) {
                                        if (v.entity.geoShape.coordinates) {
                                            geo = v.entity.geoShape.coordinates;
                                        }
                                    } else {
                                        geo = [];
                                    }
                                });
                            } else {
                                geo = [];
                            }
                            article = { src: val._source, img: val.smartnews.image, geo: geo};
                            news.push(article);
                            if (article.src.images[0] !== undefined) {
                                imgs.push(article);
                            }
                        });
                        save_hd(imgs);
                        save_repo(news);
                        set_time(new Date());
                        defer.resolve(news);
                    },
                    function () {
                        defer.resolve(undefined);
                    }
                );

                return defer.promise;
            }

            function update() {
                var defer = $q.defer(), size = 200;
                if (cs.is_smartPhone()) {
                    size = 100;
                }
                defer.resolve(get_news(size));
                return defer.promise;
            }

            function start_fetch(force, size) {
                size = size || 400;
                var defer = $q.defer(), articles = get_repo();
                if (force === true || articles === undefined || size > articles.length) {
                    if (size >= 500) {
                        size = 500;
                    }
                    defer.resolve(get_news(size));
                } else if (articles !== undefined) {
                    defer.resolve(articles);
                }
                return defer.promise;
            }

            function fetch(size) {
                size = (size || 50);
                var defer = $q.defer(), articles = get_repo();
                if (articles === undefined) {
                    defer.resolve(start_fetch(false, size));
                } else if (size >= articles.length) {
                    start_fetch(false, (size + size)).then(function () {
                        defer.resolve(get_from_repo(size));
                    });
                } else if (size < articles.length) {
                    defer.resolve(get_from_repo(size));
                }
                return defer.promise;
            }

            function news() {
                var defer = $q.defer(), size = 400;
                if (cs.is_smartPhone()) {
                    size = 100;
                }
                if (empty_db()) {
                    defer.resolve(start_fetch(size));
                } else if (passed_time()) {
                    defer.resolve(start_fetch(true, size));
                } else {
                    defer.resolve(1);
                }
                return defer.promise;
            }

            function select_img() {
                return get_hd();
            }

            function lc_art_id(id) {
                var art = get_repo(), defer = $q.defer();
                if (art !== undefined) {
                    angular.forEach(art, function (val) {
                        if (val.src.articleId === id) {
                            defer.resolve(val);
                            return defer.promise;
                        }
                    });
                }
                defer.resolve(undefined);
                return defer.promise;
            }

            function sv_art_id(id) {
                var defer = $q.defer(), art;
                news_service.get_article(id).then(function (response) {
                    if (response.length > 0) {
                        angular.forEach(response, function (val) {
                            art = { src: val._source, img: val.smartnews.image};
                            defer.resolve(art);
                        });
                    } else {
                        defer.resolve(undefined);
                    }
                });
                return defer.promise;
            }

            return {
                start_fetch: start_fetch,
                fetch: fetch,
                update: update,
                clear_repo: clear_repo,
                select_img: select_img,
                lc_art_id: lc_art_id,
                sv_art_id: sv_art_id,
                save_art: save_art,
                is_read: is_read,
                save_rate: save_rate,
                get_rate: get_rate,
                news: news,
                empty_db: empty_db,
                passed_time: passed_time,
                count_news: count_news,
                fetch_reco: fetch_reco
            };
        }]);
    app.factory('art_utility', [ function () {
        function xbreaks(s) {
            if (s !== undefined) {
                var a = s.split('');
                angular.forEach(a, function (v, k) {
                    if (v === '*' || v === '–' || (a[k - 1] === ' ' && a[k] === '-' && a[k + 1] === ' ')) {
                        a[k] = '\n';
                    }
                });
                return a.join('').toString();
            }
            return s;
        }

        function x_trim(s, e) {
            if (s) {
                if (e >= 40) {
                    return s.length > e ? s.slice(0, e) + '....' : s;
                }
                return s.slice(0, e);
            }
            return s;
        }

        return {
            xbreaks: xbreaks,
            x_trim: x_trim
        };
    }]);
}());