(function () {
    'use strict';
    var app = angular.module('smnNg');
    app.factory('request_manager', ['$http', '$q', function ($http, $q) {
        function async_request(http) {
            var defer = $q.defer();
            $http({
                method: http.method,
                url: http.xurl || ('http://smartnews.idi.ntnu.no/' + http.url),
                data: http.data

            }).
                success(function (data, status) {
                    /*console.log(http.info + ' success status: ' + status);
                     console.log(data);*/
                    defer.resolve(data);

                }).
                error(function (data, status) {
                    /*console.log(http.info + ' error status: ' + status);
                     console.log(data);*/
                    defer.reject(data);
                });
            return defer.promise;
        }

        return {
            async_request: async_request
        };
    }]);

    app.factory('store_manager', ['localStorage', function (localStorage) {

        function add_raw_store(key, item) {
            if (key && item) {
                localStorage.setItem(key, item);
            }
        }

        function get_raw_store(key) {
            if (key) {
                var raw = localStorage.getItem(key);
                if (raw === null || undefined) {
                    return undefined;
                }
                return raw;
            }
        }

        function add_to_store(key, item) {
            if (key && item) {
                item = JSON.stringify(item);
                try {
                    localStorage.setItem(key, item);
                } catch (error) {
                    console.log('..');
                } finally {
                    return true;
                }
                return true;
            }
            return false;
        }

        function get_from_store(key) {
            if (key) {
                var raw = localStorage.getItem(key);
                if (raw !== null) {
                    raw = JSON.parse(raw);
                    return raw;
                }
                return undefined;
            }
        }

        function remove_from_store(key) {
            if (key) {
                localStorage.removeItem(key);
            }
        }

        return {
            add_to_store: add_to_store,
            get_from_store: get_from_store,
            remove_from_store: remove_from_store,
            add_raw_store: add_raw_store,
            get_raw_store: get_raw_store
        };
    }]);
    app.factory('preferences', ['store_manager', 'jQuery', function (store_manager, jQuery) {
        function set_geo(loc) {
            store_manager.add_to_store('geo', loc);
        }

        function get_geo() {
            return store_manager.get_from_store('geo');
        }

        function delete_geo(key) {
            store_manager.remove_from_store(key);
        }

        function xdelete() {
            store_manager.remove_from_store('x_pref');
        }

        function xget_pref() {
            return store_manager.get_from_store('x_pref');
        }

        function xset_pref(pref) {
            var base = xget_pref();
            if (base !== undefined) {
                base = jQuery.extend({}, base, pref);
                store_manager.add_to_store('x_pref', base);
            } else {
                store_manager.add_to_store('x_pref', pref);
            }

        }

        function font_pref(size) {
            var o;
            switch (size) {
                case 1:
                    o = '0.8';
                    break;
                case 2:
                    o = '0.9';
                    break;
                case 3:
                    o = '1';
                    break;
                case 4:
                    o = '1.1';
                    break;
                case 5:
                    o = '1.2';
                    break;
                case 6:
                    o = '1.3';
                    break;
                case 7:
                    o = '1.4';
                    break;
                case 8:
                    o = '1.5';
                    break;
                case 9:
                    o = '1.6';
                    break;
                case 10:
                    o = '1.7';
                    break;
                case 11:
                    o = '1.8';
                    break;
                case 12:
                    o = '1.9';
                    break;
                case 13:
                    o = '2';
                    break;
                default:
                    o = '1';
                    break;
            }
            return o + 'em';
        }

        function bg_pref(p) {
            var a = [];
            switch (p) {
                case 1:
                    a.push({bc: 'rgb(253,233,235)', fc: '#333333'});
                    break;
                case 2:
                    a.push({bc: 'rgb(35,35,35)', fc: 'rgb(134, 185, 51)'});
                    break;
                case 3:
                    a.push({bc: 'rgb(241,241,241)', fc: '#333333'});
                    break;
                default:
                    a.push({bc: 'rgb(35,35,35)', fc: 'rgb(241,241,241)'});
                    break;
            }
            return a;
        }

        return {
            set_geo: set_geo,
            get_geo: get_geo,
            delete_geo: delete_geo,
            xset_pref: xset_pref,
            xget_pref: xget_pref,
            font_pref: font_pref,
            bg_pref: bg_pref
        };
    }]);
    app.factory('animation', ['jQuery', function (jQuery) {

        function toggle(elem, addition, size) {
            size = size || 2;
            addition = addition || '<i class="fa fa-' + size + 'x fa-refresh fa-spin"></i>';
            jQuery(elem).html('').html(addition);
        }

        function anima_i(elem, remove, add) {
            jQuery(elem).removeClass(remove).addClass(add);
        }

        return {
            toggle: toggle,
            anima_i: anima_i
        };
    }]);

    app.factory('menu', ['jQuery', function (jQuery) {
        var ctr = jQuery('#main-content'),
            lfBtn = jQuery('#left-menu-toggle'),
            lfSwipe = jQuery('.left-swipe-area'),
            rtBtn = jQuery('#right-menu-toggle'),
            rtSwipe = jQuery('.right-swipe-area'),
            toggleLeft = function () {
                jQuery('.lfnav').toggleClass('nav-left nav-open');
                lfBtn.toggleClass('mc-left-push');
                lfSwipe.toggleClass('mc-left-push');
                rtBtn.toggleClass('mc-left-push');
                rtSwipe.toggleClass('mc-left-push');
                ctr.toggleClass('mc-left-push');
            },
            toggleRight = function () {
                jQuery('.rtnav').toggleClass('nav-right nav-open');
                rtBtn.toggleClass('mc-right-push');
                rtSwipe.toggleClass('mc-right-push');
                lfBtn.toggleClass('mc-right-push');
                lfSwipe.toggleClass('mc-right-push');
                ctr.toggleClass('mc-right-push');
            };
        return {
            toggleLeft: toggleLeft,
            toggleRight: toggleRight
        };
    }]);
    app.factory('notifier', ['toastr',
        function (toastr) {

            (function setToasterOptions() {
                toastr.options.closeButton = true;
            }());

            function notify(message, type) {
                if (type === 'success') {
                    toastr.success(message);
                } else if (type === 'warning') {
                    toastr.warning(message);
                } else if (type === 'error') {
                    toastr.error(message);
                } else {
                    toastr.info(message);
                }
            }

            return {
                notify: notify
            };
        }]);

    app.factory('common_service', ['Math', 'jQuery', 'Modernizr', 'navigator', function (Math, jQuery, Modernizr, navigator) {

        var random_number = function (limit) {
                return Math.floor((Math.random() * limit));
            },
            is_touch_device = function () {
                return Modernizr.touch === true;
            },
            is_mobile_device = function () {
                return jQuery.browser.mobile === true;
            },
            is_device = {
                Android: function () {
                    return navigator.userAgent.match(/Android/i);
                },
                BlackBerry: function () {
                    return navigator.userAgent.match(/BlackBerry/i);
                },
                iOS: function () {
                    return navigator.userAgent.match(/iPhone|iPad|iPod/i);
                },
                iPhone: function () {
                    return navigator.userAgent.match(/iPhone/i);
                },
                iPad: function () {
                    return navigator.userAgent.match(/iPad/i);
                },
                Opera: function () {
                    return navigator.userAgent.match(/Opera Mini/i);
                },
                Windows: function () {
                    return navigator.userAgent.match(/IEMobile/i);
                },
                any: function () {
                    return (is_device.Android() || is_device.BlackBerry() || is_device.iOS() || is_device.Opera() || is_device.Windows());
                }
            },
            is_browser = {
                safari: function () {
                    return navigator.userAgent.match(/(iPad|iPhone|iPod touch);.*CPU.*OS 7_\d/i);
                },
                chrome: function () {
                    return navigator.userAgent.indexOf('Chrome') > -1;

                },
                firefox: function () {
                    return navigator.userAgent.indexOf('Firefox') > -1;
                },
                which_browser: function () {
                    is_browser.safari();
                    is_browser.chrome();
                    is_browser.firefox();

                }
            };

        function is_transform3d() {
            return Modernizr.csstransforms3d;
        }

        function extend(base, addons) {
            return jQuery.extend({}, base, addons);
        }

        function numbers(startRange, endRange) {
            var nums = [], i, option;
            for (i = startRange; i <= endRange; i++) {
                option = {};
                if (i < 10) {
                    option.name = '0' + i.toString();
                    option.value = '0' + i.toString();
                } else {
                    option.name = i;
                    option.value = i;
                }
                nums.push(option);
            }
            return nums;
        }

        function random_logo() {

            var logo = 'assets/img/site/default/', logos;

            if (is_mobile_device()) {
                logo = logo + 'port/';

            } else if (!is_mobile_device()) {
                logo = logo + 'desk/';
            }

            logos = ['black_red', 'black_blue', 'black_green', 'black_orange', 'gold_orange',
                'orange_black', 'white_red', 'white', 'red_white', 'black_pant'];

            return logo + logos[random_number(logos.length - 1)] + '.gif';
        }

        function random_colors() {

            var colors = ['rgb(160,236,111)', 'rgb(255,130,130)',
                'rgb(90,198,255)', 'rgb(28,154,115)', 'rgb(245,230,102)',
                'rgb(253,193,98)', 'rgb(246,141,187)', 'rgb(10,194,210)',
                'rgb(123,183,250)', 'rgb(245,222,179)', 'rgb(183,183,183)', 'rgb(134, 185, 51)'];

            return colors[random_number(colors.length - 1)];
        }

        function is_smartPhone() {
            return is_mobile_device() && is_touch_device();
        }

        return {
            random_colors: random_colors,
            random_logo: random_logo,
            is_transform3d: is_transform3d,
            extend: extend,
            is_touch_device: is_touch_device,
            is_mobile_device: is_mobile_device,
            random_number: random_number,
            is_device: is_device,
            is_browser: is_browser,
            numbers: numbers,
            is_smartPhone: is_smartPhone
        };
    }]);

    app.factory('constant_service', [
        function () {
            function numbers(startRange, endRange) {
                var nums = [], i, option;
                for (i = startRange; i <= endRange; i++) {
                    option = {};
                    if (i < 10) {
                        option.name = '0' + i.toString();
                        option.value = '0' + i.toString();
                    } else {
                        option.name = i;
                        option.value = i;
                    }
                    nums.push(option);
                }
                return nums;
            }

            function app_occ() {
                return [
                    {name: 'Other', value: 'Other'},
                    {name: 'Student', value: 'Student'},
                    {name: 'Unemployed', value: 'Unemployed'},
                    {name: 'Accounting & Auditing', value: 'Accounting & Auditing'},
                    {name: 'Administrative & Support', value: 'Administrative & Support'},
                    {name: 'Advertising, Marketing', value: 'Advertising, Marketing'},
                    {name: 'Arts, Entertainment & Media', value: 'Arts, Entertainment & Media'},
                    {name: 'Banking', value: 'Banking'},
                    {name: 'Consulting Services', value: 'Consulting Services'},
                    {name: 'Design', value: 'Design'},
                    {name: 'Engineering', value: 'Engineering'},
                    {name: 'Finance & Economics', value: 'Finance & Economics'},
                    {name: 'Human Resources', value: 'Human Resources'},
                    {name: 'Industry', value: 'Industry'},
                    {name: 'Information Technology', value: 'Information Technology'},
                    {name: 'Law Enforcement', value: 'Law Enforcement'},
                    {name: 'Manufacturing', value: 'Manufacturing'},
                    {name: 'Restaurant & Food Service', value: 'Restaurant & Food Service'},
                    {name: 'Personal Care', value: 'Personal Care'},
                    {name: 'Public Relations Jobs', value: 'Public Relations Jobs'},
                    {name: 'Sales', value: 'Sales'},
                    {name: 'Science & Research', value: 'Science & Research'},
                    {name: 'Security', value: 'Security'},
                    {name: 'Technician', value: 'Technician'},
                    {name: 'Warehousing & Distribution', value: 'Warehousing & Distribution'}
                ];
            }

            return {
                app_occ: app_occ,
                numbers: numbers
            };
        }]);
    app.factory('timer_service', [function () {
        function time(milliseconds) {
            var oneHour = 3600000,
                oneMinute = 60000,
                oneSecond = 1000,
                seconds = 0,
                minutes = 0,
                hours = 0,
                result;

            if (milliseconds >= oneHour) {
                hours = Math.floor(milliseconds / oneHour);
            }

            milliseconds = hours > 0 ? (milliseconds - hours * oneHour) : milliseconds;

            if (milliseconds >= oneMinute) {
                minutes = Math.floor(milliseconds / oneMinute);
            }

            milliseconds = minutes > 0 ? (milliseconds - minutes * oneMinute) : milliseconds;

            if (milliseconds >= oneSecond) {
                seconds = Math.floor(milliseconds / oneSecond);
            }

            milliseconds = seconds > 0 ? (milliseconds - seconds * oneSecond) : milliseconds;

            if (hours > 0) {
                result = (hours > 9 ? hours : "0" + hours) + ":";
            } else {
                result = "00:";
            }

            if (minutes > 0) {
                result += (minutes > 9 ? minutes : "0" + minutes) + ":";
            } else {
                result += "00:";
            }

            if (seconds > 0) {
                result += (seconds > 9 ? seconds : "0" + seconds);
            } else {
                result += "00";
            }
            return result;
        }

        return {
            time: time
        };
    }]);
    app.factory('page_manager', ['$anchorScroll', '$routeParams', '$location', '$window',
        function ($anchorScroll, $routeParams, $location, $window) {

            function new_page(page) {
                var url = $location.absUrl() + '/' + page;
                $window.open(url);
            }

            function anchor() {
                $anchorScroll();
            }

            function page_id() {
                return $routeParams.id;
            }

            function navigate(url) {
                $location.path(url);
            }

            function share_link(id) {
                if (page_id() !== undefined) {
                    return $location.absUrl();
                } else {
                    return $location.absUrl() + '/' + (id !== undefined ? id : '');
                }
            }

            return {
                anchor: anchor,
                page_id: page_id,
                navigate: navigate,
                share_link: share_link,
                new_page: new_page
            };
        }]);
    app.factory('modal_manager', ['jQuery', 'menu', '$timeout', function (jQuery, menu, $timeout) {
        function menu_op(scope) {
            if (jQuery('.smnrtnav, .smnlfnav').hasClass('nav-open')) {
                if (jQuery('.smnrtnav').hasClass('nav-open')) {
                    scope.right_btn.toggle_right_nav();
                    menu.toggleRight();
                }
                if (jQuery('.smnlfnav').hasClass('nav-open')) {
                    scope.left_btn.toggle_left_nav();
                    menu.toggleLeft();
                }
                return true;
            }
            return false;
        }

        function op(elm, scope) {
            var t = 0;
            if (menu_op(scope)) {
                t = 1000;
            }
            $timeout(function () {
                jQuery(elm).modal('show');
            }, t);
        }

        function op_src(scope) {
            op('#search-modal', scope);
        }

        function op_art(scope) {
            op('#xart-modal', scope);
        }

        function op_sta(scope) {
            op('#status-modal', scope);
        }

        function cl_stat() {
            jQuery('#status-modal').modal('hide');
            $timeout(function () {
                jQuery('#auth-modal').modal('show');
                $timeout(function () {
                    jQuery('#imail').focus();
                }, 500);
            }, 200);
        }

        function op_au(scope) {
            op('#auth-modal', scope);
            $timeout(function () {
                jQuery('#imail').focus();
            }, 500);
        }

        function open_profile() {

        }

        return {
            op_art: op_art,
            menu_op: menu_op,
            op_sta: op_sta,
            cl_stat: cl_stat,
            op_au: op_au,
            op_src: op_src
        };
    }]);
    app.factory('news_service', ['request_manager', function (request_manager) {

        function get_news(size) {
            var http = {method: 'GET', url: 'search?size=' + (size || ''), data: undefined, info: 'get_news'};
            return request_manager.async_request(http);
        }

        function get_paged_news(page) {
            var http = {method: 'GET', url: 'articles' + (page || ''), data: undefined, info: 'get_paged_news'};
            return request_manager.async_request(http);
        }

        function get_article(id) {
            var http = {method: 'GET', url: 'id/' + (id || ''), data: undefined, info: 'get_article'};
            return request_manager.async_request(http);
        }
        function get_reco() {
            var http = {method: 'GET', url: 'userrec', data: undefined, info: 'get_RECO'};
            return request_manager.async_request(http);
        }

        return {
            get_news: get_news,
            get_paged_news: get_paged_news,
            get_article: get_article,
            get_reco: get_reco
        };
    }]);
    app.factory('news_manager', ['$q', 'news_service', 'store_manager', 'common_service',
        function ($q, news_service, store_manager, cs) {
            function fetch_page(page) {
                var defer = $q.defer();
                news_service.get_paged_news(page).then(
                    function (response) {
                        response.next = response.paging.next.substring(response.paging.next.indexOf('?'));
                        if (response.paging.prev !== undefined) {
                            response.prev = response.paging.prev.substring(response.paging.prev.indexOf('?'));
                        }
                        defer.resolve(response);
                    },
                    function () {
                        defer.resolve(undefined);
                    }
                );
                return defer.promise;
            }

            function fetch_reco() {
                var defer = $q.defer();
                news_service.get_reco().then(
                    function (response) {
                        defer.resolve(response);
                    },
                    function () {
                        defer.resolve(undefined);
                    }
                );
                return defer.promise;
            }

            function clear_repo() {
                store_manager.remove_from_store('r_nx');
            }

            function save(key, news) {
                store_manager.add_to_store(key, news);
            }

            function get(key) {
                return store_manager.get_from_store(key);
            }

            function count(key) {
                var articles = store_manager.get_from_store(key);
                return articles === undefined ? 0 : articles.length;
            }

            function count_news() {
                return count('r_nx');
            }

            function save_repo(news) {
                save('r_nx', news);
            }

            function get_repo() {
                return get('r_nx');
            }

            function empty_db() {
                return get_repo() === undefined;
            }

            function save_read(news) {
                save('r_ax', news);
            }

            function get_read() {
                return get('r_ax');
            }

            function clear_read() {
                store_manager.remove_from_store('r_ax');
            }

            function save_hd(news) {
                save('h_nx', news);
            }

            function get_hd() {
                return get('h_nx');
            }

            function set_time(time) {
                save('rx_t', time);
            }

            function get_time() {
                return get('rx_t');
            }

            function passed_time() {
                var ot = get_time(), nt = new Date().getTime();
                if (ot !== undefined) {
                    ot = new Date(ot);
                    ot = nt - ot;
                    if (ot > (3600000 * 6)) {
                        return true;
                    }
                    return false;
                }
                return false;
            }

            function save_art(xid) {
                var arts = get_read(), ob = {id: xid}, ar = [], f = false;
                if (arts === undefined) {
                    ar.push(ob);
                    save_read(ar);
                } else if (arts !== undefined) {
                    angular.forEach(arts, function (v) {
                        if (v.id === ob.id) {
                            f = true;
                        }
                    });
                    if (!f) {
                        arts.push(ob);
                        save_read(arts);
                    }
                }
            }

            function save_rate(axid, rate) {
                var arts = get_read();
                angular.forEach(arts, function (v) {
                    if (v.id === axid) {
                        v.rate = rate;
                        save_read(arts);
                        return;
                    }

                });
            }

            function get_rate(axid) {
                var arts = get_read(), r = 0;
                angular.forEach(arts, function (v) {
                    if (v.id === axid) {
                        if (v.rate !== undefined) {
                            r = v.rate;
                        }
                    }
                });
                return r;
            }

            function is_read(id) {
                var arts = get_read(), f = false;
                if (arts !== undefined) {
                    angular.forEach(arts, function (v) {
                        if (v.id === id) {
                            f = true;
                        }
                    });
                }
                return f;
            }

            function get_from_repo(size) {
                var articles = get_repo(), result = [], i;
                if (articles !== undefined) {
                    for (i = (size - 50); i < size; i = (i + 1)) {
                        if (articles[i] !== undefined) {
                            result.push(articles[i]);
                        }
                    }
                    return result;
                }
                return undefined;
            }

            function get_news(size) {
                var news = [], imgs = [], defer = $q.defer(), article, geo = [], shape;
                news_service.get_news(size).then(
                    function (response) {
                        angular.forEach(response, function (val) {
                            if (val._source.entities !== undefined && val._source.entities.length > 0) {
                                shape = val._source.entities;
                                angular.forEach(shape, function (v) {
                                    if (v.entity.geoShape) {
                                        if (v.entity.geoShape.coordinates) {
                                            geo = v.entity.geoShape.coordinates;
                                        }
                                    } else {
                                        geo = [];
                                    }
                                });
                            } else {
                                geo = [];
                            }
                            article = { src: val._source, img: val.smartnews.image, geo: geo};
                            news.push(article);
                            if (article.src.images[0] !== undefined) {
                                imgs.push(article);
                            }
                        });
                        save_hd(imgs);
                        save_repo(news);
                        set_time(new Date());
                        defer.resolve(news);
                    },
                    function () {
                        defer.resolve(undefined);
                    }
                );

                return defer.promise;
            }

            function update() {
                var defer = $q.defer(), size = 200;
                if (cs.is_smartPhone()) {
                    size = 100;
                }
                defer.resolve(get_news(size));
                return defer.promise;
            }

            function start_fetch(force, size) {
                size = size || 400;
                var defer = $q.defer(), articles = get_repo();
                if (force === true || articles === undefined || size > articles.length) {
                    if (size >= 500) {
                        size = 500;
                    }
                    defer.resolve(get_news(size));
                } else if (articles !== undefined) {
                    defer.resolve(articles);
                }
                return defer.promise;
            }

            function fetch(size) {
                size = (size || 50);
                var defer = $q.defer(), articles = get_repo();
                if (articles === undefined) {
                    defer.resolve(start_fetch(false, size));
                } else if (size >= articles.length) {
                    start_fetch(false, (size + size)).then(function () {
                        defer.resolve(get_from_repo(size));
                    });
                } else if (size < articles.length) {
                    defer.resolve(get_from_repo(size));
                }
                return defer.promise;
            }

            function news() {
                var defer = $q.defer(), size = 400;
                if (cs.is_smartPhone()) {
                    size = 100;
                }
                if (empty_db()) {
                    defer.resolve(start_fetch(size));
                } else if (passed_time()) {
                    defer.resolve(start_fetch(true, size));
                } else {
                    defer.resolve(1);
                }
                return defer.promise;
            }

            function select_img() {
                return get_hd();
            }

            function lc_art_id(id) {
                var art = get_repo(), defer = $q.defer();
                if (art !== undefined) {
                    angular.forEach(art, function (val) {
                        if (val.src.articleId === id) {
                            defer.resolve(val);
                            return defer.promise;
                        }
                    });
                }
                defer.resolve(undefined);
                return defer.promise;
            }

            function sv_art_id(id) {
                var defer = $q.defer(), art;
                news_service.get_article(id).then(function (response) {
                    if (response.length > 0) {
                        angular.forEach(response, function (val) {
                            art = { src: val._source, img: val.smartnews.image};
                            defer.resolve(art);
                        });
                    } else {
                        defer.resolve(undefined);
                    }
                });
                return defer.promise;
            }

            return {
                start_fetch: start_fetch,
                fetch: fetch,
                update: update,
                clear_repo: clear_repo,
                select_img: select_img,
                lc_art_id: lc_art_id,
                sv_art_id: sv_art_id,
                save_art: save_art,
                is_read: is_read,
                save_rate: save_rate,
                get_rate: get_rate,
                news: news,
                empty_db: empty_db,
                passed_time: passed_time,
                count_news: count_news,
                fetch_reco: fetch_reco
            };
        }]);
    app.factory('art_utility', [ function () {
        function xbreaks(s) {
            if (s !== undefined) {
                var a = s.split('');
                angular.forEach(a, function (v, k) {
                    if (v === '*' || v === '–' || (a[k - 1] === ' ' && a[k] === '-' && a[k + 1] === ' ')) {
                        a[k] = '\n';
                    }
                });
                return a.join('').toString();
            }
            return s;
        }

        function x_trim(s, e) {
            if (s) {
                if (e >= 40) {
                    return s.length > e ? s.slice(0, e) + '....' : s;
                }
                return s.slice(0, e);
            }
            return s;
        }

        return {
            xbreaks: xbreaks,
            x_trim: x_trim
        };
    }]);
    app.factory('event_service', ['request_manager', function (request_manager) {
        function post_events(events) {
            var http = {method: 'POST', url: 'events', data: events, info: 'post_events'};
            return request_manager.async_request(http);
        }

        return {
            post_events: post_events
        };
    }]);

    app.factory('event_manager', ['$q', 'event_service', 'identity_manager', 'store_manager',
        function ($q, event_service, identity_manager, store_manager) {
            function save_rating(r) {
                store_manager.add_to_store('x_rate', r);
            }

            function get_rating_store() {
                return store_manager.get_from_store('x_rate');
            }

            function clean_rating_store() {
                store_manager.remove_from_store('x_rate');
            }

            function post_events(user, events) {
                var defer = $q.defer(), identity;
                if (user !== undefined && events.length > 0) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        identity.events = events;
                        event_service.post_events(identity).
                            then(
                            function (response) {
                                defer.resolve(response);
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function register_ratings(user, events) {
                var defer = $q.defer();
                post_events(user, events).
                    then(function (response) {
                        if (response.error === 0 && response.nevents === events.length) {
                            defer.resolve(true);
                        } else {
                            defer.resolve(false);
                        }
                    }, function () {
                        defer.resolve(false);
                    });
                return defer.promise;
            }

            function add_all_ratings(user) {
                var ratings = get_rating_store();
                register_ratings(user, ratings).then(function (result) {
                    if (result) {
                        clean_rating_store();
                    }
                    return result;
                });
            }

            function add_rating_store(user, rating) {
                var ratings = get_rating_store(), s = true;
                if (ratings === undefined) {
                    ratings = [];
                    ratings.push(rating);
                    save_rating(ratings);
                } else {
                    angular.forEach(ratings, function (v) {
                        if (v.article_id === rating.article_id) {
                            if (v.rating !== rating.rating) {
                                v.rating = rating.rating;
                                save_rating(ratings);
                            }
                            s = false;
                        }
                    });
                    if (s) {
                        ratings.push(rating);
                        save_rating(ratings);
                    }
                    if (ratings.length >= 20) {
                        add_all_ratings(user);
                    }
                }
            }

            function add_rating(user, rating) {
                var defer = $q.defer(), identity;
                if (user !== undefined && rating !== undefined) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        add_rating_store(user, rating);
                        defer.resolve(true);
                    }
                } else {
                    defer.resolve(false);
                }
                return defer.promise;
            }

            return {
                add_rating: add_rating
            };
        }]);
    app.factory('user_service', ['request_manager', function (request_manager) {

        function authenticate(user) {
            var http = {method: 'POST', url: 'auth', data: user, info: 'authenticate_user'};
            return request_manager.async_request(http);
        }

        function register(user) {
            var http = {method: 'POST', url: 'register', data: user, info: 'register_user'};
            return request_manager.async_request(http);
        }

        function logout(user) {
            var http = {method: 'GET', url: 'logout', data: user, info: 'logout_user'};
            return request_manager.async_request(http);
        }

        function get_profile(user) {
            var http = {method: 'GET', url: 'profile', data: user, info: 'GET: PROFILE'};
            return request_manager.async_request(http);
        }

        function set_profile(profile) {
            var http = {method: 'POST', url: 'profile', data: profile, info: 'SET: PROFILE'};
            return request_manager.async_request(http);
        }
        function get_prefs(user) {
            var http = {method: 'GET', url: 'prefs', data: user, info: 'GET: PREFS'};
            return request_manager.async_request(http);
        }

        function set_prefs(profile) {
            var http = {method: 'POST', url: 'prefs', data: profile, info: 'SET: PREFS'};
            return request_manager.async_request(http);
        }


        function get_location() {
            var http = {method: 'JSONP', xurl: 'http://ipinfo.io?callback=JSON_CALLBACK', data: undefined, info: 'get_location'};
            return request_manager.async_request(http);
        }

        return {
            authenticate: authenticate,
            register: register,
            logout: logout,
            get_profile: get_profile,
            set_profile: set_profile,
            get_prefs: get_prefs,
            set_prefs: set_prefs,
            get_location: get_location
        };
    }]);
    app.factory('user_manager', ['$q', 'user_service', 'identity_manager', 'common_service', 'md5',
        function ($q, user_service, identity_manager, common_service, md5) {

            function handle_auth_res(response, user) {
                if (response.error === 0) {
                    user.addons = {image_url: 'https://s.gravatar.com/avatar/' + md5(user.email) + '?s=100'};
                    var authenticated = common_service.extend(user, response);
                    identity_manager.set_user_identity(authenticated);
                    return authenticated;
                }
                return undefined;
            }

            function handle_response(response) {
                if (response.error === 0) {
                    return 1;
                }
                return -1;
            }

            function identify_user() {
                return identity_manager.get_user_identity();
            }

            function authenticate_user(user) {
                var defer = $q.defer();
                if (user !== undefined) {
                    if (user.email !== undefined && user.password !== undefined) {
                        user_service.authenticate(user).
                            then(
                            function (response) {
                                defer.resolve(handle_auth_res(response, user));
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function register_user(user) {
                var defer = $q.defer();
                if (user !== undefined) {
                    if (user.email !== undefined && user.password !== undefined) {
                        user.registration_date = new Date();
                        user_service.register(user).
                            then(
                            function (response) {
                                defer.resolve(handle_auth_res(response, user));
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function logout_user() {
                var defer = $q.defer(),
                    identity = identity_manager.get_user_identity();
                if (identity !== undefined) {
                    user_service.logout(identity).
                        then(
                        function (response) {
                            if (response.error === 0) {
                                identity_manager.remove_user_identity();
                                defer.resolve(true);
                            }
                        },
                        function (response) {
                            defer.reject(response);
                        }
                    );
                }
                return defer.promise;
            }

            function get_user_profile(user) {
                var defer = $q.defer(), identity;
                if (user !== undefined) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        user_service.get_profile(identity).
                            then(
                            function (response) {
                                defer.resolve(response);
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function set_user_profile(user, profile) {

                var defer = $q.defer(), identity;
                if (user !== undefined && profile !== undefined && Object.keys(profile).length > 0) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        identity.profile = profile;
                        user_service.set_profile(identity).
                            then(
                            function (response) {
                                defer.resolve(handle_response(response));
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function get_user_prefs(user) {
                var defer = $q.defer(), identity;
                if (user !== undefined) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        user_service.get_prefs(identity).
                            then(
                            function (response) {
                                defer.resolve(response);
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }
            function set_user_prefs(user, prefs) {
                var defer = $q.defer(), identity;
                if (user !== undefined && prefs !== undefined && Object.keys(prefs).length > 0) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        identity.prefs = prefs;
                        user_service.set_prefs(identity).
                            then(
                            function (response) {
                                defer.resolve(handle_response(response));
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function current_loc() {
                var defer = $q.defer();
                user_service.get_location().then(
                    function (r) {
                        var c = r.loc.split(','),
                            l = { city: r.city, region: r.region,
                                country: r.country, latitude: c[0], longitude: c[1]};
                        defer.resolve(l);
                    },
                    function () {
                        defer.resolve(undefined);
                    }
                );
                return defer.promise;
            }

            return {
                identify_user: identify_user,
                authenticate_user: authenticate_user,
                register_user: register_user,
                logout_user: logout_user,
                get_user_profile: get_user_profile,
                set_user_profile: set_user_profile,
                get_user_prefs: get_user_prefs,
                set_user_prefs: set_user_prefs,
                current_loc: current_loc
            };
        }]);
    app.factory('identity_manager', ['$cookieStore', function ($cookieStore) {
        function set_user_identity(identity) {
            console.log(identity);
            delete identity.password;
            delete identity.email;

            console.log(identity)
            if (identity !== undefined && identity !== null) {
                $cookieStore.put('identity', identity);
            }
        }

        function get_user_identity() {
            var identity = $cookieStore.get('identity');
            if (identity !== undefined) {
                return identity;
                return identity;
            }
            return undefined;
        }

        function site_requester_identify() {
            return get_user_identity();
        }

        function remove_user_identity() {
            $cookieStore.remove('identity');
        }

        function approval_of_identity(user) {
            var identity = site_requester_identify();
            if (identity !== undefined) {
                if ((user.uid === identity.uid)/* &&
                 (user.email === identity.email) /* &&
                 (user.password) === (identity.password)*/) {
                    return identity;
                }
            }
            return undefined;
        }

        return {
            site_requester_identify: site_requester_identify,
            set_user_identity: set_user_identity,
            get_user_identity: get_user_identity,
            remove_user_identity: remove_user_identity,
            approval_of_identity: approval_of_identity
        };
    }]);
}());