/*
 These methods should not be called by any other call than the manager class.
 */
(function () {
    'use strict';
    var app = angular.module('smnNg');
    app.factory('user_service', ['request_manager', function (request_manager) {

        function authenticate(user) {
            var http = {method: 'POST', url: 'auth', data: user, info: 'authenticate_user'};
            return request_manager.async_request(http);
        }

        function register(user) {
            var http = {method: 'POST', url: 'register', data: user, info: 'register_user'};
            return request_manager.async_request(http);
        }

        function logout(user) {
            var http = {method: 'GET', url: 'logout', data: user, info: 'logout_user'};
            return request_manager.async_request(http);
        }

        function get_profile(user) {
            var http = {method: 'GET', url: 'profile', data: user, info: 'GET: PROFILE'};
            return request_manager.async_request(http);
        }

        function set_profile(profile) {
            var http = {method: 'POST', url: 'profile', data: profile, info: 'SET: PROFILE'};
            return request_manager.async_request(http);
        }
        function get_prefs(user) {
            var http = {method: 'GET', url: 'prefs', data: user, info: 'GET: PREFS'};
            return request_manager.async_request(http);
        }

        function set_prefs(profile) {
            var http = {method: 'POST', url: 'prefs', data: profile, info: 'SET: PREFS'};
            return request_manager.async_request(http);
        }


        function get_location() {
            var http = {method: 'JSONP', xurl: 'http://ipinfo.io?callback=JSON_CALLBACK', data: undefined, info: 'get_location'};
            return request_manager.async_request(http);
        }

        return {
            authenticate: authenticate,
            register: register,
            logout: logout,
            get_profile: get_profile,
            set_profile: set_profile,
            get_prefs: get_prefs,
            set_prefs: set_prefs,
            get_location: get_location
        };
    }]);
    app.factory('user_manager', ['$q', 'user_service', 'identity_manager', 'common_service', 'md5',
        function ($q, user_service, identity_manager, common_service, md5) {

            function handle_auth_res(response, user) {
                if (response.error === 0) {
                    user.addons = {image_url: 'https://s.gravatar.com/avatar/' + md5(user.email) + '?s=100'};
                    var authenticated = common_service.extend(user, response);
                    identity_manager.set_user_identity(authenticated);
                    return authenticated;
                }
                return undefined;
            }

            function handle_response(response) {
                if (response.error === 0) {
                    return 1;
                }
                return -1;
            }

            function identify_user() {
                return identity_manager.get_user_identity();
            }

            function authenticate_user(user) {
                var defer = $q.defer();
                if (user !== undefined) {
                    if (user.email !== undefined && user.password !== undefined) {
                        user_service.authenticate(user).
                            then(
                            function (response) {
                                defer.resolve(handle_auth_res(response, user));
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function register_user(user) {
                var defer = $q.defer();
                if (user !== undefined) {
                    if (user.email !== undefined && user.password !== undefined) {
                        user.registration_date = new Date();
                        user_service.register(user).
                            then(
                            function (response) {
                                defer.resolve(handle_auth_res(response, user));
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function logout_user() {
                var defer = $q.defer(),
                    identity = identity_manager.get_user_identity();
                if (identity !== undefined) {
                    user_service.logout(identity).
                        then(
                        function (response) {
                            if (response.error === 0) {
                                identity_manager.remove_user_identity();
                                defer.resolve(true);
                            }
                        },
                        function (response) {
                            defer.reject(response);
                        }
                    );
                }
                return defer.promise;
            }

            function get_user_profile(user) {
                var defer = $q.defer(), identity;
                if (user !== undefined) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        user_service.get_profile(identity).
                            then(
                            function (response) {
                                defer.resolve(response);
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function set_user_profile(user, profile) {

                var defer = $q.defer(), identity;
                if (user !== undefined && profile !== undefined && Object.keys(profile).length > 0) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        identity.profile = profile;
                        user_service.set_profile(identity).
                            then(
                            function (response) {
                                defer.resolve(handle_response(response));
                            },
                            function (response) {
                                defer.reject(response);
                            }
                        );
                    }
                }
                return defer.promise;
            }

            function get_user_prefs(user) {
                var defer = $q.defer(), identity;
                if (user !== undefined) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        user_service.get_prefs(identity).
                            then(
                                function (response) {
                                    defer.resolve(response);
                                },
                                function (response) {
                                    defer.reject(response);
                                }
                            );
                    }
                }
                return defer.promise;
            }
            function set_user_prefs(user, prefs) {
                var defer = $q.defer(), identity;
                if (user !== undefined && prefs !== undefined && Object.keys(prefs).length > 0) {
                    identity = identity_manager.approval_of_identity(user);
                    if (identity !== undefined) {
                        identity.prefs = prefs;
                        user_service.set_prefs(identity).
                            then(
                                function (response) {
                                    defer.resolve(handle_response(response));
                                },
                                function (response) {
                                    defer.reject(response);
                                }
                            );
                    }
                }
                return defer.promise;
            }

            function current_loc() {
                var defer = $q.defer();
                user_service.get_location().then(
                    function (r) {
                        var c = r.loc.split(','),
                            l = { city: r.city, region: r.region,
                                country: r.country, latitude: c[0], longitude: c[1]};
                        defer.resolve(l);
                    },
                    function () {
                        defer.resolve(undefined);
                    }
                );
                return defer.promise;
            }

            return {
                identify_user: identify_user,
                authenticate_user: authenticate_user,
                register_user: register_user,
                logout_user: logout_user,
                get_user_profile: get_user_profile,
                set_user_profile: set_user_profile,
                get_user_prefs: get_user_prefs,
                set_user_prefs: set_user_prefs,
                current_loc: current_loc
            };
        }]);
    app.factory('identity_manager', ['$cookieStore', function ($cookieStore) {
        function set_user_identity(identity) {
            console.log(identity);
            delete identity.password;
            delete identity.email;

            console.log(identity)
            if (identity !== undefined && identity !== null) {
                $cookieStore.put('identity', identity);
            }
        }

        function get_user_identity() {
            var identity = $cookieStore.get('identity');
            if (identity !== undefined) {
                return identity;
                return identity;
            }
            return undefined;
        }

        function site_requester_identify() {
            return get_user_identity();
        }

        function remove_user_identity() {
            $cookieStore.remove('identity');
        }

        function approval_of_identity(user) {
            var identity = site_requester_identify();
            if (identity !== undefined) {
                if ((user.uid === identity.uid)/* &&
                    (user.email === identity.email) /* &&
                    (user.password) === (identity.password)*/) {
                    return identity;
                }
            }
            return undefined;
        }

        return {
            site_requester_identify: site_requester_identify,
            set_user_identity: set_user_identity,
            get_user_identity: get_user_identity,
            remove_user_identity: remove_user_identity,
            approval_of_identity: approval_of_identity
        };
    }]);
}());